
/*****************************************************************************/
/*                                                                           */
/*   Program : Cut Processor of Plug-In-able Switching System                */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 10/17/1999                                                    */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"

/* Declare functions */
void *Switchlet(void *);
void CtrlHndr(int , struct StSwletCmd *);
void DataHndr(int , struct StSwletCmd *);

/* Define Global variables */
int giErrLogFd;
char gszErrLog[MAX_LOGSTRING_SIZE];

#define DEFAULT_CUT_SIZE 10


main(argc, argv)
int argc;
char *argv[];
{
	int iSkFd;
	int iLength;
	int iAcceptFd;
	pthread_t iThread;
	struct sockaddr_un stRcvAddr;

	Init(&iSkFd, argv[0]);
	
	for (;;) {
		iLength = sizeof(stRcvAddr);
		if ((iAcceptFd = accept(iSkFd, (struct sockaddr *)&stRcvAddr, &iLength)) < 0) {
			sprintf(gszErrLog, "Error in accept.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			close(giErrLogFd);
			close(iSkFd);
			exit(1);
		}
		iThread++;
		if (iThread >= 100000)
			iThread = 0;	/* Reset iThread */
		if (pthread_create(&iThread, NULL, Switchlet, (void *)(&iAcceptFd))) {
			sprintf(gszErrLog, "Error in Create Hndr (thread index %d)\n", iThread);
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		}
		pthread_detach(iThread);
	}
}

int
Init(piSkFd, szLogName)
int *piSkFd;
char *szLogName;
{
	struct sockaddr_un stSkAddr;
	char szName[MAX_FILENAME_SIZE];


	/* make log file name */
	strcpy(szName, szLogName);
	strcat(szName, ".log");
	
	if ((giErrLogFd = open(szName, O_WRONLY | O_APPEND | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error Can't open ErrLog file.\n");
		exit(1);
	}

	/* make Sk */
	stSkAddr.sun_family = AF_LOCAL;
	strcpy(szName, "Sk");
	strcat(szName, szLogName);
	unlink(szName);

	if ((*piSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in create Sk.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		exit(1);
	}

	strcpy(stSkAddr.sun_path, szName);
	if (bind(*piSkFd, (struct sockaddr *)&stSkAddr, sizeof(stSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in bind.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		close(*piSkFd);
		exit(1);
	}
	listen(*piSkFd, BACK_LOG);
	return OK;
}

/* Switchlet */
void *
Switchlet(arg)
void *arg;
{
	int iFd;
	struct StSwletCmd stSwletCmd;

	iFd = *((int *)arg);
	
	if (read(iFd, &stSwletCmd, sizeof(stSwletCmd)) != sizeof(stSwletCmd)) {
		sprintf(gszErrLog, "Error in Read First SwletCmd.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		return (void *) NULL;
	}

	switch (stSwletCmd.ulCId) {
	case SWLET_CTRL :
		CtrlHndr(iFd, &stSwletCmd);
		break;
	case SWLET_DATA :
		DataHndr(iFd, &stSwletCmd);
		break;
	case SWLET_KILL :
		sprintf(gszErrLog, "Switchlet Terminated.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		close(giErrLogFd);
		exit(0);
	default :
		sprintf(gszErrLog, "Error in Control Id %ld\n", stSwletCmd.ulCId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	close(iFd);
	pthread_exit(NULL);
	return (void *)NULL;
}

void
CtrlHndr(iFd, pstSwletCmd)
int iFd;
struct StSwletCmd *pstSwletCmd;
{
	unsigned long *pulSize;
	struct stSwletCmd;

	if (pstSwletCmd->ulCId == SWLET_CTRL && pstSwletCmd->iShmId < 0) {
		/* Session Inf does not exist */
		sprintf(gszErrLog, "Session Info does not Exist. (CtrlHndr)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	} else {
		/* Session Inf exist */
		sprintf(gszErrLog, "Session Info Exist. (CtrlHndr)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	}

	close(iFd);
	return;
}

void
DataHndr(iFd, pstSwletCmd)
int iFd;
struct StSwletCmd *pstSwletCmd;
{
	int iShmId, iSsInfShmId;
	int iNewShmId, iTmpShmId;
	struct StPkInf *pstPkInf;
	unsigned char *psNewPacketData, *psTmpPacketData;
	unsigned long *pulCutSize;
	unsigned long ulCutSize;
	int i;

	if (pstSwletCmd->iShmId < 0) {	/* Session Inf does not exist */
		ulCutSize = DEFAULT_CUT_SIZE;
	} else {
		if ((pulCutSize = shmat(pstSwletCmd->iShmId, 0, 0)) == (unsigned long *) -1) {
			sprintf(gszErrLog, "Error in shmat.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			close(iFd);
			return;
		}
		/* Session Inf exist */
		ulCutSize = *pulCutSize;
		ulCutSize = ntohl(ulCutSize);
		shmdt(pulCutSize);
	}

	if (read(iFd, &iShmId, sizeof(iShmId)) != sizeof(iShmId)) {	/* read Packet Inf ShmId */
		sprintf(gszErrLog, "Error in Read Packet Inf ShmId.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		return;
	}

	if ((pstPkInf = shmat(iShmId, 0, 0)) == (struct StPkInf *) -1) {
		sprintf(gszErrLog, "Error in Packet Inf shmat.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		return;
	}

	if (pstPkInf->stPkHdr.ulPkSize < ulCutSize) {
		ulCutSize = pstPkInf->stPkHdr.ulPkSize;
	}

	if ((pstPkInf->psPkData = shmat(pstPkInf->iShmId, 0, 0)) == (unsigned char *) -1) { /* read Packet Data ShmId */
		sprintf(gszErrLog, "Error in Packet Data shmat.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkInf);
		close(iFd);
		return;
	}

	if ((iNewShmId = shmget(IPC_PRIVATE, ulCutSize, SHM_R | SHM_W)) < 0) { /* get new shm for cut */
		sprintf(gszErrLog, "Error in shmget.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkInf->psPkData);
		shmdt(pstPkInf);
		close(iFd);
		return;
	}

	if ((psNewPacketData = shmat(iNewShmId, 0, 0)) == (unsigned char *) -1) {
		sprintf(gszErrLog, "Error in shmat....\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmctl(iNewShmId, IPC_RMID, 0);
		shmdt(pstPkInf->psPkData);
		shmdt(pstPkInf);
		close(iFd);
		return;
	}

	/* Copy old packet to new packet */
	memcpy(psNewPacketData, pstPkInf->psPkData, ulCutSize);

	pstPkInf->stPkHdr.ulPkSize = ulCutSize;	/* assign new packet size */

	iTmpShmId = pstPkInf->iShmId;
	psTmpPacketData = pstPkInf->psPkData;

	pstPkInf->iShmId = iNewShmId;
	pstPkInf->psPkData = psNewPacketData;

	pstPkInf->ulHndrId = RT_HNDR;	/* assign next handler */

	sprintf(gszErrLog, "Processed.SrcId %ld Session %ld Seq %ld.\n", pstPkInf->stPkHdr.ulSrcId, pstPkInf->stPkHdr.ulSsId, pstPkInf->stPkHdr.ulSeqNo);
	write(giErrLogFd, gszErrLog, strlen(gszErrLog));

	shmdt(psTmpPacketData);
	shmctl(iTmpShmId, IPC_RMID, 0);

	shmdt(pstPkInf->psPkData);
	shmdt(pstPkInf);
	write(iFd, &iShmId, sizeof(iShmId));
}

