/*****************************************************************************/
/*                                                                           */
/*   Program : CutSwlet of Plug-In-able Switching System                     */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "CutSwlet.h"

#define CutSwletErrLogName	"CutSwletErr.log"

int giCutSwletErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];

int
CutSwletInit(szTxtSwNameIpAddr, iCtrlPort, ulSrcId, ulSsId, piTcpSkFd)
char *szTxtSwNameIpAddr;
int iCtrlPort;
unsigned long ulSrcId, ulSsId;
int *piTcpSkFd;
{

	if ((giCutSwletErrLogFd = open(CutSwletErrLogName, O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		PutCutSwletSts("Error Log files can not access.");
		return ERROR;
	}

	PutCutSwletSts("CutSwlet Ctrl Session Open.");

	if (OpenCutSwletCtrlSession(szTxtSwNameIpAddr, iCtrlPort, ulSrcId, ulSsId, piTcpSkFd) == ERROR) {
		close(giCutSwletErrLogFd);
		return ERROR;
	}

	return OK;
}

int
OpenCutSwletCtrlSession(szTxtSwNameIpAddr, iCtrlPort, ulSrcId, ulSsId, piTcpSkFd)
char *szTxtSwNameIpAddr;
int iCtrlPort;
unsigned long ulSrcId, ulSsId;
int *piTcpSkFd;
{
	int i;
	struct sockaddr_in stSsCtrlSkAddr;
	struct StSsCtrlData stSsCtrlData;

	*piTcpSkFd = socket(AF_INET, SOCK_STREAM, 0);
	
	bzero(&stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));
	stSsCtrlSkAddr.sin_family = AF_INET;
	stSsCtrlSkAddr.sin_port = htons(iCtrlPort);
	inet_pton(AF_INET, szTxtSwNameIpAddr, &stSsCtrlSkAddr.sin_addr);

	connect(*piTcpSkFd, (struct sockaddr *) &stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));

	stSsCtrlData.ulCcId = (SESSION_CTRL);
	stSsCtrlData.ulSrcId = ulSrcId;
	stSsCtrlData.ulSsId = ulSsId;
	
	HtoNstSsCtrlData(&stSsCtrlData, &stSsCtrlData);
	if (write(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData)) < 0) {
		sprintf(gszErrLog, "Error in sending Session Create.\n");
		write(giCutSwletErrLogFd, gszErrLog, strlen(gszErrLog));
		PutCutSwletSts(gszErrLog);
		close(*piTcpSkFd);
		return ERROR;
	}

	i = read(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData));
	
	NtoHstSsCtrlData(&stSsCtrlData, &stSsCtrlData);

	if (stSsCtrlData.ulCcId != SESSION_CTRL_ACK) {
		sprintf(gszErrLog, "Error in receiving Prc Create Ack.\n");
		write(giCutSwletErrLogFd, gszErrLog, strlen(gszErrLog));
		PutCutSwletSts(gszErrLog);
		close(*piTcpSkFd);
		return ERROR;
	}
	return OK;
}

int
SendCutSwletCtrlToSession(szTxtSwNameIpAddr, ulCutSize, iTcpSkFd)
char *szTxtSwNameIpAddr;
unsigned long ulCutSize;
int iTcpSkFd;
{
	unsigned long ulTmp;
	struct StSwletCmd stSwletCmd;

	ulTmp = htonl(sizeof(ulCutSize));
	write(iTcpSkFd, &ulTmp, sizeof(ulTmp));

	ulCutSize = htonl(ulCutSize);
	write(iTcpSkFd, &ulCutSize, sizeof(ulCutSize));

	return OK;
}

int
CloseCutSwletSession(iTcpSkFd)
int iTcpSkFd;
{
	close(iTcpSkFd);
	close(giCutSwletErrLogFd);
	return OK;
}
