
/*****************************************************************************/
/*                                                                           */
/*   Program : Manager Component of Plug-In-able Switching System            */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "ManComp.h"

/* define global variable */
extern int giErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];
extern int giCtrlPort;
extern int giDataPort;

/* Supervisor */
void *
ManagerSupervisor(arg)
void *arg;
{
	int iTcpSkFd;
	int iUdpSkFd;
	int iChildPid;
	pthread_t iTid;


	MakeTcpPort(&iTcpSkFd);
	MakeUdpPort(&iUdpSkFd);
	iTid = 0;
	if (pthread_create(&iTid, NULL, TcpPortMonitor, (void *)(&iTcpSkFd))) {
		sprintf(gszErrLog, "Error in create thread for TcpPortMonitor.(Supervisor)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	}
	pthread_detach(iTid);
	iTid++;

	UdpPortMonitor((void *)(&iUdpSkFd));
	pthread_exit(NULL);
	return (void *)NULL;
}

/* Resource Manager */
int
ResourceManager()
{
	return OK;
}

/* Component Download Manager */
int
ComponentDownloadManager()
{
	return OK;
}

/* Send Ack to Sender */
int
SendAck(iTcpPortFd, pstSsCtrlData)
int iTcpPortFd;
struct StSsCtrlData *pstSsCtrlData;
{
	struct StSsCtrlData stTmpSsCtrlData;
	switch (pstSsCtrlData->ulCcId) {
	case SESSION_CREATE :
	case SESSION_CREATE_ACK :
		stTmpSsCtrlData.ulCcId = SESSION_CREATE_ACK;
		stTmpSsCtrlData.ulSsId = pstSsCtrlData->ulSsId;
		HtoNstSsCtrlData(&stTmpSsCtrlData, &stTmpSsCtrlData);
		write(iTcpPortFd, &stTmpSsCtrlData, sizeof(stTmpSsCtrlData));
	case SESSION_RELEASE :
	case SESSION_RELEASE_ACK :
		stTmpSsCtrlData.ulCcId = SESSION_RELEASE_ACK;
		stTmpSsCtrlData.ulSsId = pstSsCtrlData->ulSsId;
		HtoNstSsCtrlData(&stTmpSsCtrlData, &stTmpSsCtrlData);
		write(iTcpPortFd, &stTmpSsCtrlData, sizeof(stTmpSsCtrlData));
		break;
	case SESSION_CTRL :
	case SESSION_CTRL_ACK :
		stTmpSsCtrlData.ulCcId = SESSION_CTRL_ACK;
		stTmpSsCtrlData.ulSsId = pstSsCtrlData->ulSsId;
		HtoNstSsCtrlData(&stTmpSsCtrlData, &stTmpSsCtrlData);
		write(iTcpPortFd, &stTmpSsCtrlData, sizeof(stTmpSsCtrlData));
		break;
	default :
		sprintf(gszErrLog, "Error in SendAck (ulCcId = %d)\n", pstSsCtrlData->ulCcId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	return OK;
}

/* Tcp Port Handler */
void *
TcpPortHandler(arg)
void *arg;
{
	int iTcpPortSkFd;
	struct StSsCtrlData stSsCtrlData;
	
	iTcpPortSkFd = *((int *)(arg));
	if (read(iTcpPortSkFd, &stSsCtrlData, sizeof(stSsCtrlData)) != sizeof(stSsCtrlData)) {
		sprintf(gszErrLog, "Error in Reading SsCtrlData. (size mismatch).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iTcpPortSkFd);
		pthread_exit(NULL);
	}
	
	NtoHstSsCtrlData(&stSsCtrlData, &stSsCtrlData);

	switch (stSsCtrlData.ulCcId) {

	case SESSION_CTRL :
	case SESSION_CTRL_ACK :
		SessionManager(iTcpPortSkFd, &stSsCtrlData);
		break;
	case PROCESS_CREATE :
	case PROCESS_CREATE_ACK :
	case PROCESS_RELEASE :
	case PROCESS_RELEASE_ACK :
	case PROCESS_CTRL :
	case PROCESS_CTRL_ACK :
		SwletManager(iTcpPortSkFd, &stSsCtrlData);
		break;
	default :
		sprintf(gszErrLog, "Error in TcpPortHandler (ulCcId = %d)\n", stSsCtrlData.ulCcId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	close(iTcpPortSkFd);
	pthread_exit(NULL);

	return ((void *)NULL);
}

/* Udp Port Handler */
void *
UdpPortHandler(arg)
void *arg;
{
	struct StPk *pstPacket;
	int i;
	

	pstPacket = (struct StPk *)(arg);

	NtoHstPkHdr(&(pstPacket->stPkHdr), &(pstPacket->stPkHdr));

	i = PacketManager(pstPacket);

	free(pstPacket);

	return ((void *)NULL);
}

/* Tcp Port Monitor */
void *
TcpPortMonitor(arg)
void *arg;
{
	int iTcpSkFd;
	struct sockaddr_in stClTcpSkAddr;
	int iClTcpSkLength;
	int iTcpConnectFd;
	pthread_t iThread;

	iTcpSkFd = *((int *)arg);
	iThread = 0;

	for (;;) {
		iClTcpSkLength = sizeof(stClTcpSkAddr);
		iTcpConnectFd = accept(iTcpSkFd, (struct sockaddr *)&stClTcpSkAddr, &iClTcpSkLength);

		iThread++;
		if (iThread > 100000) iThread = 0;	/* reset iThread */
		if (pthread_create(&iThread, NULL, TcpPortHandler, (void *)(&iTcpConnectFd))) {
			sprintf(gszErrLog, "Error in Create Tcp Port Monitor (thread index %d)\n", iThread);
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			PutMsg(gszErrLog);
		}
		pthread_detach(iThread);
	}
}

/* Udp Port Monitor */
void *
UdpPortMonitor(arg)
void *arg;
{
	int iUdpSkFd;
	struct sockaddr stClUdpSkAddr;
	struct StPk *pstPacket;
	int iClUdpSkLength;
	
	iUdpSkFd = *((int *)arg);

	for (;;) {
		if ((pstPacket = (struct StPk *)malloc(sizeof(*pstPacket))) == NULL) {
			sprintf(gszErrLog, "Error in allocating pstPacket (UdpPortMonitor).\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		} else {
			iClUdpSkLength = sizeof(stClUdpSkAddr);
			if (recvfrom(iUdpSkFd, pstPacket, sizeof(*pstPacket), 0, &stClUdpSkAddr, &iClUdpSkLength) <= 0) {
				sprintf(gszErrLog, "Error in recvfrom (UdpPortMonitor).\n");
				write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			} else {
				UdpPortHandler((void *)(pstPacket));
			}
		}
	}
}

/* Make Tcp Port */
int
MakeTcpPort(piTcpSkFd)
int *piTcpSkFd;
{
	struct sockaddr_in stSvrTcpSkAddr;
	
	*piTcpSkFd = socket(AF_INET, SOCK_STREAM, 0);
	
	bzero(&stSvrTcpSkAddr, sizeof(stSvrTcpSkAddr));
	stSvrTcpSkAddr.sin_family = AF_INET;
	stSvrTcpSkAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	stSvrTcpSkAddr.sin_port = htons(giCtrlPort);
	
	bind(*piTcpSkFd, (struct sockaddr *)&stSvrTcpSkAddr, sizeof(stSvrTcpSkAddr));
	
	listen(*piTcpSkFd, BACK_LOG);

	return OK;
}

/* Make Udp Port */
int
MakeUdpPort(piUdpSkFd)
int *piUdpSkFd;
{
	struct sockaddr_in stSvrUdpSkAddr;
	
	*piUdpSkFd = socket(AF_INET, SOCK_DGRAM, 0);
	
	bzero(&stSvrUdpSkAddr, sizeof(stSvrUdpSkAddr));
	stSvrUdpSkAddr.sin_family = AF_INET;
	stSvrUdpSkAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	stSvrUdpSkAddr.sin_port = htons(giDataPort);
	
	bind(*piUdpSkFd, (struct sockaddr *)&stSvrUdpSkAddr, sizeof(stSvrUdpSkAddr));

	return OK;
}

