
/*****************************************************************************/
/*                                                                           */
/*   Program : Process Component of Plug-In-able Switching System            */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "ProcComp.h"

/* declare global variable */
extern int giErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];
extern struct StSsCmpo astSsCmpo[MAX_SESSION_NO];

/* define global variable */
static struct StPrcTbl astPrcTbl[MAX_PROCESS_NO];

/* Process Supervisor */
void *
ProcessSupervisor(arg)
void *arg;
{
	int i;
	struct StPkInf *pstPkInf;

	PutMsg("Running PrcSupervisor.\n");
	for (;;) {
		for (i=0; i < MAX_SESSION_NO; i++) {
			if (IsSsExist(i)) {
				if (IsPkExist(i)) {
					SwletActivate(GetPkCmpo(i));
					if ((pstPkInf = shmat(PkInfShmId(i), 0, 0)) == (struct StPkInf *) -1) {
						sprintf(gszErrLog, "Error in PkInf shmat.(PrcActivate).\n");
						write(giErrLogFd, gszErrLog, strlen(gszErrLog));
					} else {
						if (pstPkInf->ulHndrId == RT_HNDR) {
							shmdt(pstPkInf);
							PacketSend(GetPkCmpo(i));
							PacketFree(GetPkCmpoAddr(i));
						} else {
							shmdt(pstPkInf);
						}
					}
				} else if (!IsSsAlive(i)) {
					ReleaseSession(i);
				}
			}
		}
	}
}

/* Swlet Activate */
int
SwletActivate(pstPkCmpo)
struct StPkCmpo *pstPkCmpo;
{
	int iIndex;
	int iSkFd;
	int iShmId;
	unsigned long ulSrcId, ulSsId, ulSeqNo;
	struct StPkInf *pstPkInf;
	struct sockaddr_un stClSkAddr;
	struct StSwletCmd stSwletCmd;

	if ((pstPkInf = shmat(pstPkCmpo->iShmId, 0, 0)) == (struct StPkInf *) -1) {
		sprintf(gszErrLog, "Error in shmat.(PrcActivate).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	if ((iIndex = SwletTblFind(pstPkInf->ulHndrId)) == ERROR) {
		sprintf(gszErrLog, "Error in SwletTblFind.%d(PrcActivate).\n",pstPkInf->ulHndrId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkInf);
		return ERROR;
	}
	ulSrcId = pstPkInf->stPkHdr.ulSrcId;
	ulSsId = pstPkInf->stPkHdr.ulSsId;
	ulSeqNo = pstPkInf->stPkHdr.ulSeqNo;

	shmdt(pstPkInf);

	if ((iSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in OpenSk.(PrcActivate).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	bzero(&stClSkAddr, sizeof(stClSkAddr));
	stClSkAddr.sun_family = AF_LOCAL;
	MkSwletSkName(stClSkAddr.sun_path, iIndex);

	if (connect(iSkFd, (struct sockaddr *)&stClSkAddr, sizeof(stClSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in connect Sk. (PrcActivate).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	stSwletCmd.ulCId = SWLET_DATA;
	stSwletCmd.ulCSize = sizeof(iShmId);
	stSwletCmd.iShmId = GetSsInf(ulSrcId, ulSsId);

	write(iSkFd, &stSwletCmd, sizeof(stSwletCmd));
	write(iSkFd, &pstPkCmpo->iShmId, sizeof(pstPkCmpo->iShmId));

	read(iSkFd, &iShmId, sizeof(iShmId));
	close(iSkFd);
	sprintf(gszErrLog, "Activate. Src/SsId/SeqNo(%ld:%ld:%ld)(PrcActivate).\n", ulSrcId, ulSsId, ulSeqNo);
	write(giErrLogFd, gszErrLog, strlen(gszErrLog));

	return OK;
}

/* Swlet Manager */
int
SwletManager(iTcpPortSkFd, pstSsCtrlData)
int iTcpPortSkFd;
struct StSsCtrlData *pstSsCtrlData;
{
	switch (pstSsCtrlData->ulCcId) {
	case PROCESS_CREATE :
		if (SwletCreate(iTcpPortSkFd, pstSsCtrlData) == ERROR) {
			sprintf(gszErrLog, "Error in SwletCreate.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			PutMsg(gszErrLog);
			return ERROR;
		}
		break;
	case PROCESS_CREATE_ACK :
		sprintf(gszErrLog, "Not implemented PROCESS_CREATE_ACK in SwletManager.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	case PROCESS_RELEASE :
		if (SwletRelease(pstSsCtrlData) == ERROR) {
			sprintf(gszErrLog, "Error in SwletRelease.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			return ERROR;
		}
		break;
	case PROCESS_RELEASE_ACK :
		sprintf(gszErrLog, "Not implemented PROCESS_RELEASE_ACK in SwletManager.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	case PROCESS_CTRL :
	case PROCESS_CTRL_ACK :
		if (SwletCtrl(iTcpPortSkFd, pstSsCtrlData) == ERROR) {
			sprintf(gszErrLog, "Error in PrcCtl.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			return ERROR;
		}
		break;
	default :
		sprintf(gszErrLog, "Error in SwletManager (ulCcId = %d).\n", pstSsCtrlData->ulCcId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	return OK;
}

/* Swlet Create */
int
SwletCreate(iTcpPortSkFd, pstSsCtrlData)
int iTcpPortSkFd;
struct StSsCtrlData *pstSsCtrlData;
{
	int iReadSize;
	int iTblIndex;
	int iExcuTblCodeFd;
	struct StPkHdr stPkHdr;
	struct StSsCtrlData stTmpSsCtrlData;
	unsigned char sReadBuffer[MAX_BUFFER_SIZE];

	stTmpSsCtrlData.ulCcId = PROCESS_CREATE_ACK;	/* send ack */
	stTmpSsCtrlData.ulSsId = 0;
	HtoNstSsCtrlData(&stTmpSsCtrlData, &stTmpSsCtrlData);
	write(iTcpPortSkFd, &stTmpSsCtrlData, sizeof(stTmpSsCtrlData));

	if ((iReadSize = read(iTcpPortSkFd, &stPkHdr, sizeof(stPkHdr))) != sizeof(stPkHdr)) {
		sprintf(gszErrLog, "Error in reading packet header. (SwletCreate)\n");
		write(giErrLogFd, gszErrLog, sizeof (gszErrLog));
		return ERROR;
	}
	NtoHstPkHdr(&stPkHdr, &stPkHdr);

	if ((iTblIndex = SwletTblFind(stPkHdr.ulDataId)) == ERROR) {
		if ((iTblIndex = SwletTblFindEmpty()) == ERROR) {
			sprintf(gszErrLog, "Error in SwletCreate.(No Empty PrcTbl entry)\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			PutMsg(gszErrLog);
			return ERROR;
		}
	} else {
		SwletTblRelease(iTblIndex);
	}

	if (SwletTblAllocate(iTblIndex, &stPkHdr) == ERROR) {
		return ERROR;
	}

	if ((iExcuTblCodeFd = open((const char *)astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IROTH)) == -1) {
		sprintf(gszErrLog, "Error in open Prc file %s.\n", astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	while ((iReadSize = read(iTcpPortSkFd, sReadBuffer, sizeof(sReadBuffer))) > 0) {
		write(iExcuTblCodeFd, sReadBuffer, iReadSize);
	}
	
	close(iExcuTblCodeFd);

	if (fork() == 0) {	/* child */
		SwletRun(astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc);
		exit(0);
	}
	return OK;
}

/* Swlet Release */
int
SwletRelease(pstSsCtrlData)
struct StSsCtrlData *pstSsCtrlData;
{
	int i;
	if ( (i = SwletFind(pstSsCtrlData->ulSsId)) != ERROR) {
		SwletTblRelease(i);
		return OK;
	}
	sprintf(gszErrLog, "Error Can't find PrcId(%d) in PrcTbl. (SwletRelease)\n", pstSsCtrlData->ulSsId);
	write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	return ERROR;
}

/* Swlet Table Allocate */
int
SwletTblAllocate(iTblIndex, pstPkHdr)
int iTblIndex;
struct StPkHdr *pstPkHdr;
{
	char szFileName[MAX_FILENAME_SIZE];

	astPrcTbl[iTblIndex].ulDataType = pstPkHdr->ulDataId;
	if ((astPrcTbl[iTblIndex].pstPrcCmpo = (struct StPrcCmpo *)malloc(sizeof(struct StPrcCmpo))) == NULL) {
		sprintf(gszErrLog, "Error in allocatinng stPrcComponent.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	astPrcTbl[iTblIndex].pstPrcCmpo->ulPrcId = pstPkHdr->ulDataId;
	bzero(&astPrcTbl[iTblIndex].pstPrcCmpo->stPrcInf, sizeof(astPrcTbl[iTblIndex].pstPrcCmpo->stPrcInf));
	sprintf(szFileName, "Prc%ld.pcs", astPrcTbl[iTblIndex].pstPrcCmpo->ulPrcId);
	PutMsg("Switchlet Load : ");
	PutMsg(szFileName);
	PutMsg("\n");
	if ((astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc = malloc(strlen(szFileName)+1)) == NULL) {
		sprintf(gszErrLog, "Error in allocatinng pszExecCodeLoc.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	strcpy(astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc, szFileName);

	return OK;
}

/* Swlet Table Release */
int
SwletTblRelease(iTblIndex)
int iTblIndex;
{
	SwletKill(iTblIndex);
	free(astPrcTbl[iTblIndex].pstPrcCmpo->pszExecCodeLoc);
	free(astPrcTbl[iTblIndex].pstPrcCmpo);
	astPrcTbl[iTblIndex].ulDataType = 0;
	astPrcTbl[iTblIndex].pstPrcCmpo = NULL;

	return OK;
}

/* Swlet Run */
int
SwletRun(pszName)
char *pszName;
{
	char szName[MAX_FILENAME_SIZE];
	strcpy(szName, getcwd(NULL, MAX_FILENAME_SIZE));
	strcat(szName, "/");
	strcat(szName, pszName);
	execl(szName, pszName, NULL);
	return OK;
}

/* Swlet Find */
int
SwletFind(ulPrcId)
unsigned long ulPrcId;
{
	int i;
	
	for (i = 0; i < MAX_PROCESS_NO; i++) {
		if (astPrcTbl[i].pstPrcCmpo->ulPrcId == ulPrcId) {
			return i;
		}
	}
	return ERROR;
}

/* Swlet Tbl Find */
int
SwletTblFind(ulHndrId)
unsigned long ulHndrId;
{
	int i;

	for (i = 0; i < MAX_PROCESS_NO; i++) {
		if (astPrcTbl[i].ulDataType == ulHndrId) {
			return i;
		}
	}
	return ERROR;
}

/* Swlet Tbl Find Empty */
int
SwletTblFindEmpty()
{
	int i;
	for (i = 0; i < MAX_PROCESS_NO; i++) {
		if (astPrcTbl[i].ulDataType == 0) {
			return i;
		}
	}
	return ERROR;
}

/* Switchlet Kill */
int
SwletKill(i)
int i;
{
	int iSkFd;
	struct sockaddr_un stClSkAddr;
	struct StSwletCmd stSwletCmd;

	if ((iSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in OpenSk.(SwletKill).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	bzero(&stClSkAddr, sizeof(stClSkAddr));
	stClSkAddr.sun_family = AF_LOCAL;
	MkSwletSkName(stClSkAddr.sun_path, i);

	if (connect(iSkFd, (struct sockaddr *)&stClSkAddr, sizeof(stClSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in connect Sk. (SwletKill).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	stSwletCmd.ulCId = SWLET_KILL;
	stSwletCmd.ulCSize = 0;
	stSwletCmd.iShmId = -1;

	write(iSkFd, &stSwletCmd, sizeof(stSwletCmd));
	close(iSkFd);
	return OK;
}

/* Switchlet Control */
int
SwletCtrl(iTcpSkFd, pstSsCtrlData)
int iTcpSkFd;
struct StSsCtrlData *pstSsCtrlData;
{
	int iIndex;
	int iSwletSkFd;
	struct sockaddr_un stClSkAddr;
	struct StSwletCmd stSwletCmd;
	struct StSsCtrlData stTmpSsCtrlData;
	unsigned char sBuf[MAX_BUFFER_SIZE];
	int i;
	
	if ((iIndex = SwletTblFind(pstSsCtrlData->ulSsId)) == ERROR) {
		sprintf(gszErrLog, "Error in SwletTblFind.%d(SwletCtrl).\n",pstSsCtrlData->ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	if ((iSwletSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in OpenSk.(SwletCtrl).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	stTmpSsCtrlData.ulCcId = PROCESS_CTRL_ACK;	/* send ack */
	stTmpSsCtrlData.ulSsId = pstSsCtrlData->ulSsId;
	HtoNstSsCtrlData(&stTmpSsCtrlData, &stTmpSsCtrlData);
	write(iTcpSkFd, &stTmpSsCtrlData, sizeof(stTmpSsCtrlData));

	bzero(&stClSkAddr, sizeof(stClSkAddr));
	stClSkAddr.sun_family = AF_LOCAL;
	MkSwletSkName(stClSkAddr.sun_path,iIndex);
	if (connect(iSwletSkFd, (struct sockaddr *)&stClSkAddr, sizeof(stClSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in connect Sk. (SwletCtrl).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	read(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));	/* get swletcmd */
	NtoHstSwletCmd(&stSwletCmd, &stSwletCmd);
	stSwletCmd.iShmId = GetSsInf(pstSsCtrlData->ulSrcId, pstSsCtrlData->ulSsId);

	PutMsg("Receiving Swletcmd.\n");
	while (stSwletCmd.ulCId != SWLET_END) {
		write(iSwletSkFd, &stSwletCmd, sizeof(stSwletCmd));
		if (stSwletCmd.ulCSize > 0) {
			i = read(iTcpSkFd, sBuf, stSwletCmd.ulCSize);
			write(iSwletSkFd, sBuf, i);
		}
		read(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));
		NtoHstSwletCmd(&stSwletCmd, &stSwletCmd);
	}
	write(iSwletSkFd, &stSwletCmd, sizeof(stSwletCmd));

	PutMsg("End of Swletcmd.\n");
	read(iSwletSkFd, &stSwletCmd, sizeof(stSwletCmd));
	SetSsInf(pstSsCtrlData->ulSrcId, pstSsCtrlData->ulSsId, stSwletCmd.iShmId);
	HtoNstSwletCmd(&stSwletCmd, &stSwletCmd);
	write(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));
	close(iSwletSkFd);

	return OK;
}

/* Get Executable Name */
char *
GetExecName(ulHndrId)
unsigned long ulHndrId;
{
	int i;

	if ((i = SwletTblFind(ulHndrId)) == ERROR) {
		sprintf(gszErrLog, "Error in SwletTblFind. %d(GetExecName).\n", ulHndrId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return NULL;
	}
	return astPrcTbl[i].pstPrcCmpo->pszExecCodeLoc;
}

/* Get Executable Name List */
int
GetExecNameList(szStr)
char *szStr;
{
	int i;

	for (i=0; i < MAX_PROCESS_NO; i++) {
		if (astPrcTbl[i].ulDataType > 0) {
			strcat(szStr, astPrcTbl[i].pstPrcCmpo->pszExecCodeLoc);
			strcat(szStr,"\n");
		}
	}
	return OK;
}
