
/*****************************************************************************/
/*                                                                           */
/*   Program : Routing Switchlet of Plug-In-able Switching System            */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 10/17/1999                                                    */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"

/* Define Constants */
#define MAX_RT_SIZE 100
#define szSysIdFileName "Sys.Id"

/* Define Types */
struct StRt {
	unsigned long ulName;
	unsigned long ulRtAddr;
	unsigned long ulCtrlPort;
	unsigned long ulDataPort;
};

/* Define Global variables */
int giErrLogFd;
char gszErrLog[MAX_LOGSTRING_SIZE];


static struct StRt stRt[MAX_RT_SIZE];
static unsigned long ulSwId;



/* Declare functions */
void *
Switchlet(void *);

void
CtrlHandler(int, struct StSwletCmd *);

void
DataHandler(int);

struct StRt *
FindRTVacant(struct StRt **);

struct StRt *
FindRT(unsigned long);

struct StRt *
GetRTE(unsigned long);

main(argc, argv)
int argc;
char *argv[];
{
	int iSkFd;
	int iLength;
	int iAcceptFd;
	pthread_t iThread;
	struct sockaddr_un stRcvAddr;

	Init(&iSkFd, argv[0]);
	
	for (;;) {
		iLength = sizeof(stRcvAddr);
		if ((iAcceptFd = accept(iSkFd, (struct sockaddr *)&stRcvAddr, &iLength)) < 0) {
			sprintf(gszErrLog, "Error in accept.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			close(giErrLogFd);
			close(iSkFd);
			exit(1);
		}
		iThread++;
		if (pthread_create(&iThread, NULL, Switchlet, (void *)(&iAcceptFd))) {
			sprintf(gszErrLog, "Error in Create Handler (thread index %d)\n", iThread);
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		}
		pthread_detach(iThread);
	}
}

int
Init(piSkFd, szLogName)
int *piSkFd;
char *szLogName;
{
	struct sockaddr_un stSkAddr;
	char szName[MAX_FILENAME_SIZE];
	char szSwId[50];
	FILE *SwIdFd;

	/* initilize local variable */
	bzero(stRt, sizeof(stRt));

	/* make log file name */
	strcpy(szName, szLogName);
	strcat(szName, ".log");
	
	if ((giErrLogFd = open(szName, O_WRONLY | O_APPEND | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error Can't open ErrLog file.\n");
		exit(1);
	}

	/* read Switch Id */
	if ((SwIdFd = fopen(szSysIdFileName, "r")) == NULL) {
		sprintf(gszErrLog, "Error in open Id file%s.\n", szSysIdFileName);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		exit(1);
	}
	fscanf(SwIdFd, "%s", szSwId);
	ulSwId = strtoul(szSwId, (char **)NULL, 10);
/*
	printf("Switch Id is %ld\n", ulSwId);
*/
	fclose(SwIdFd);

	/* make Sk */
	stSkAddr.sun_family = AF_LOCAL;
	strcpy(szName, "Sk");
	strcat(szName, szLogName);
	unlink(szName);

	if ((*piSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in create Sk.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		exit(1);
	}

	strcpy(stSkAddr.sun_path, szName);
	if (bind(*piSkFd, (struct sockaddr *)&stSkAddr, sizeof(stSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in bind.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		close(*piSkFd);
		exit(1);
	}
	listen(*piSkFd, BACK_LOG);
	return OK;
}

/* Switchlet */
void *
Switchlet(arg)
void *arg;
{
	int iFd;
	struct StSwletCmd stSwletCmd;

	iFd = *((int *)arg);
	if (read(iFd, &stSwletCmd, sizeof(stSwletCmd)) != sizeof(stSwletCmd)) {
		sprintf(gszErrLog, "Error in Read SwletCmd.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		return (void *) NULL;
	}

	switch (stSwletCmd.ulCId) {
	case SWLET_CTRL :
/*
		printf("SWLET_CTRL\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case RTC_TBL_INIT :
/*
		printf("RTC_TBL_INIT\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case RTC_TBL_LOAD :
/*
		printf("RTC_TBL_LOAD\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case RTC_TBL_FLUSH :
/*
		printf("RTC_TBL_FLUSH\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case RTC_ETR_ADD :
/*
		printf("RTC_TBL_ADD\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case RTC_ETR_REMOVE :
/*
		printf("RTC_TBL_REMOVE\n");
*/
		CtrlHandler(iFd, &stSwletCmd);
		break;
	case SWLET_DATA :
		DataHandler(iFd);
		break;
	default :
		sprintf(gszErrLog, "Error in Control Id (%d,%d)\n", stSwletCmd.ulCId, stSwletCmd.ulCSize);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	close(iFd);
	pthread_exit(NULL);
	return (void *)NULL;
}

void
CtrlHandler(iFd, pstSwletCmd)
int iFd;
struct StSwletCmd *pstSwletCmd;
{
	unsigned long ulName;
	struct {
		unsigned long ulSwId;
		unsigned long ulName;
		unsigned long ulRtAddr;
		unsigned long ulCtrlPort;
		unsigned long ulDataPort;
	} stTmpRt;
	struct StRt *pstRt;
	char szBuf[MAX_BUFFER_SIZE];
	int i;

/*
	printf("CtrlHandler.\n");
*/

	if (read(iFd, pstSwletCmd, sizeof(*pstSwletCmd)) != sizeof(*pstSwletCmd)) {
		sprintf(gszErrLog, "Error in Read SwletCmd..\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iFd);
		return;
	}

	switch (pstSwletCmd->ulCId) {
	case RTC_TBL_INIT :
		bzero(stRt, sizeof(stRt));
		pstSwletCmd->ulCId = OK;
		pstSwletCmd->ulCSize = 0;
		write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		break;
	case RTC_TBL_LOAD :
/*
		printf("RTC_TBL_LOAD.\n");
*/
		bzero(stRt, sizeof(stRt));
		i = 0;
		while (pstSwletCmd->ulCId != SWLET_END) {
			read(iFd, &stTmpRt, sizeof(stTmpRt));
			stTmpRt.ulSwId = ntohl(stTmpRt.ulSwId);
			stTmpRt.ulName = ntohl(stTmpRt.ulName);
			stTmpRt.ulRtAddr = ntohl(stTmpRt.ulRtAddr);
			stTmpRt.ulCtrlPort = ntohl(stTmpRt.ulCtrlPort);
			stTmpRt.ulDataPort = ntohl(stTmpRt.ulDataPort);
			if (stTmpRt.ulSwId == ulSwId) {
				stRt[i].ulName = stTmpRt.ulName;
				stRt[i].ulRtAddr = stTmpRt.ulRtAddr;
				stRt[i].ulCtrlPort = stTmpRt.ulCtrlPort;
				stRt[i].ulDataPort = stTmpRt.ulDataPort;
				i++;
/*
				printf("Accept SwId %d Rt %d. Addr %x, CtrlPort %ld, DataPort %ld.\n", stTmpRt.ulSwId, stTmpRt.ulName, stTmpRt.ulRtAddr, stTmpRt.ulCtrlPort, stTmpRt.ulDataPort);
			} else {
				printf("SwId %d Rt %d. Addr %x, CtrlPort %ld, DataPort %ld.\n", stTmpRt.ulSwId, stTmpRt.ulName, stTmpRt.ulRtAddr, stTmpRt.ulCtrlPort, stTmpRt.ulDataPort);
*/
			}
			if (read(iFd, pstSwletCmd, sizeof(*pstSwletCmd)) != sizeof(*pstSwletCmd)) {
				sprintf(gszErrLog, "Error in Read SwletCmd...\n");
				write(giErrLogFd, gszErrLog, strlen(gszErrLog));
				close(iFd);
				return;
			}
		}
		pstSwletCmd->ulCId = OK;
		pstSwletCmd->ulCSize = 0;
		write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));

/*
		while (i >= 0)
			printf("Routing Table[%d] Name %ld Addr %x CtrlPort %ld, DataPort %ld\n",i--, stRt[i].ulName, stRt[i].ulRtAddr, stRt[i].ulCtrlPort, stRt[i].ulDataPort);
*/
		break;
	case RTC_TBL_FLUSH :
		bzero(stRt, sizeof(stRt));
		pstSwletCmd->ulCId = OK;
		pstSwletCmd->ulCSize = 0;
		write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		break;
	case RTC_ETR_ADD :
/*
		printf("Before ADD.\n");
*/
		pstRt = NULL;
		if (FindRTVacant(&pstRt) == NULL) {
/*
			printf("Error return.\n");
*/
			sprintf(gszErrLog, "Error in FindRTVacant.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			pstSwletCmd->ulCId = NOK;
			pstSwletCmd->ulCSize = 0;
			write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		} else {
/*
			printf("Before return.\n");
*/
			read(iFd, pstRt, sizeof(*pstRt));
			pstRt->ulName = ntohl(pstRt->ulName);
			pstRt->ulRtAddr = ntohl(pstRt->ulRtAddr);
			pstRt->ulCtrlPort = ntohl(pstRt->ulCtrlPort);
			pstRt->ulDataPort = ntohl(pstRt->ulDataPort);
/*
			printf("Rt %ld %x (%ld,%ld) is added.\n", pstRt->ulName, pstRt->ulRtAddr, pstRt->ulCtrlPort, pstRt->ulDataPort);
			printf("After read .\n");
*/
			pstSwletCmd->ulCId = OK;
			pstSwletCmd->ulCSize = 0;
/*
			printf("Before write .\n");
*/
			write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
/*
			printf("Rt %d %x (%ld,%ld) is added.\n", pstRt->ulName, pstRt->ulRtAddr, pstRt->ulCtrlPort, pstRt->ulDataPort);
*/
/*
			UltoSzulRtgInf(szBuf, pstRt->ulRtAddr);
			sprintf(gszErrLog, "RT %d, %s is added.\n", pstRt->ulName, szBuf);
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
*/
		}
		break;
	case RTC_ETR_REMOVE :
		read(iFd, &ulName, pstSwletCmd->ulCSize);
		if ((pstRt = FindRT(ulName)) == NULL) {
			sprintf(gszErrLog, "Error in FindRT.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			pstSwletCmd->ulCId = NOK;
			pstSwletCmd->ulCSize = 0;
			write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		} else {
			bzero(pstRt, sizeof(*pstRt));
			pstSwletCmd->ulCId = OK;
			pstSwletCmd->ulCSize = 0;
			write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		}
		break;
	default :
		pstSwletCmd->ulCId = NOK;
		pstSwletCmd->ulCSize = 0;
		write(iFd, pstSwletCmd, sizeof(*pstSwletCmd));
		sprintf(gszErrLog, "Error in Cid.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	
}

void
DataHandler(iFd)
int iFd;
{
	struct StRt stRtE;
	struct StRt *pstRt;
	struct StSwletCmd stSwletCmd;

	if (read(iFd, &stRtE.ulName, sizeof(stRtE.ulName)) != sizeof(stRtE.ulName)) {
		sprintf(gszErrLog, "Error in Read ulName.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		stSwletCmd.ulCId = NOK;
		stSwletCmd.ulCSize = 0;
		write(iFd, &stSwletCmd, sizeof(stSwletCmd));
		return;
	}
	if ((pstRt = GetRTE(stRtE.ulName)) == NULL) {
		sprintf(gszErrLog, "Error in Get ulName.%ld\n", stRtE.ulName);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		stSwletCmd.ulCId = NOK;
		stSwletCmd.ulCSize = 0;
		write(iFd, &stSwletCmd, sizeof(stSwletCmd));
		return;
	}
/*
	printf("Routing Addr %x, CtrlPort %ld, DataPort %ld.\n", pstRt->ulRtAddr, pstRt->ulCtrlPort, pstRt->ulDataPort);
*/
	stSwletCmd.ulCId = OK;
	stSwletCmd.ulCSize = 0;
	write(iFd, &stSwletCmd, sizeof(stSwletCmd));
	write(iFd, pstRt, sizeof(*pstRt));
}


struct StRt *
FindRTVacant(ppstRt)
struct StRt **ppstRt;
{
	int i;
	
	for (i=0; i < MAX_RT_SIZE; i++) {
		if (stRt[i].ulName == 0) {
/*
			printf("Found %d.\n", i);
*/
			*ppstRt = &(stRt[i]);
			return *ppstRt;
		}
	}
	return NULL;
}

struct StRt *
FindRT(ulName)
unsigned long ulName;
{
	int i;
	
	for (i=0; i < MAX_RT_SIZE; i++) {
		if (stRt[i].ulName == ulName) {
/*
			printf("Found i=%d, %ld\n", i, ulName);
*/
			return &stRt[i];
		}
	}
	return NULL;
}

struct StRt *
GetRTE(ulName)
unsigned long ulName;
{
	struct StRt *pstRt;
	if ((pstRt = FindRT(ulName)) == NULL)
		return NULL;
	else {
/*
		printf("Addr %x CtrlPort %ld, DataPort %ld.\n", pstRt->ulRtAddr, pstRt->ulCtrlPort, pstRt->ulDataPort);
*/
		return pstRt;
	}
}


