
/*****************************************************************************/
/*                                                                           */
/*   Program : RtgCtrl of Plug-In-able Switching System                      */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "RtgCtrl.h"

#define RtgCtrlErrLog	"RtgCtrl.log"

int giRtgCtrlErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];

int
RtgCtrlInit(szTxtSwInfo, pSwInfoFd)
char *szTxtSwInfo;
FILE **pSwInfoFd;
{
	if ((giRtgCtrlErrLogFd = open(RtgCtrlErrLog, O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		PutRtgCtrlSts("Error Log files can not access.");
		return ERROR;
	}


	if ((*pSwInfoFd = fopen(szTxtSwInfo, "r")) == NULL) {
		PutRtgCtrlSts("Error in Open Sw Info file.");
		return ERROR;
	}

	return OK;
}

int
OpenRtgCtrlSession(szSwName, pulSsId, piTcpSkFd)
char *szSwName;
unsigned long *pulSsId;
int *piTcpSkFd;
{
	int i;
	struct sockaddr_in stSsCtrlSkAddr;
	struct StSsCtrlData stSsCtrlData;
	char szSwIpAddr[50];
	int iCtrlPort, iDataPort;

	GetAddrInf(szSwName, szSwIpAddr, &iCtrlPort, &iDataPort);
	*pulSsId = 0;
	
	*piTcpSkFd = socket(AF_INET, SOCK_STREAM, 0);
	
	bzero(&stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));
	stSsCtrlSkAddr.sin_family = AF_INET;
	stSsCtrlSkAddr.sin_port = htons(iCtrlPort);
	inet_pton(AF_INET, szSwIpAddr, &stSsCtrlSkAddr.sin_addr);

	connect(*piTcpSkFd, (struct sockaddr *) &stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));

	stSsCtrlData.ulCcId = (PROCESS_CTRL);
	stSsCtrlData.ulSsId = RT_HNDR;
	
	HtoNstSsCtrlData(&stSsCtrlData, &stSsCtrlData);
	if (write(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData)) < 0) {
		sprintf(gszErrLog, "Error in sending Session Create.\n");
		write(giRtgCtrlErrLogFd, gszErrLog, strlen(gszErrLog));
		PutRtgCtrlSts(gszErrLog);
		close(*piTcpSkFd);
		return ERROR;
	}

	i = read(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData));
	
	NtoHstSsCtrlData(&stSsCtrlData, &stSsCtrlData);

/*
	printf("ulCcId = %d.\n", stSsCtrlData.ulCcId);
*/
	
	if (stSsCtrlData.ulCcId != PROCESS_CTRL_ACK) {
		sprintf(gszErrLog, "Error in receiving Prc Create Ack.\n");
		write(giRtgCtrlErrLogFd, gszErrLog, strlen(gszErrLog));
		PutRtgCtrlSts(gszErrLog);
		close(*piTcpSkFd);
		return ERROR;
	}
	return OK;
}

int
SendRtgCtrlToSession(szTxtRtgTbl, iTcpSkFd)
char *szTxtRtgTbl;
int iTcpSkFd;
{
	struct StSwletCmd stSwletCmd;
	struct StRt {
		unsigned long ulSwName;
		unsigned long ulName;
		unsigned long ulRtAddr;
		unsigned long ulCtrlPort;
		unsigned long ulDataPort;
	} stRt;
	FILE *RtgTblFd;
	char szBuf[256];
	char szSwName[50], szDstName[50], szToName[50], szIpAddr[50];
	char *pszStr;
	int iCtrlPort, iDataPort;
	
	if ((RtgTblFd = fopen(szTxtRtgTbl, "r")) == NULL) {
		sprintf(gszErrLog, "Error in Open Rt Info File.\n");
		write(giRtgCtrlErrLogFd, gszErrLog, strlen(gszErrLog));
		PutRtgCtrlSts(gszErrLog);
		return ERROR;
	}
	
	stSwletCmd.ulCId = SWLET_CTRL;
	stSwletCmd.ulCSize = 0;
	HtoNstSwletCmd(&stSwletCmd, &stSwletCmd);
	write(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));

	while (!feof(RtgTblFd)) {
		stSwletCmd.ulCId = RTC_TBL_LOAD;
		stSwletCmd.ulCSize = sizeof(stRt);
		HtoNstSwletCmd(&stSwletCmd, &stSwletCmd);
		write(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));

		fgets(szBuf, 256, RtgTblFd);
		while ((szBuf[0] == '#') && !feof(RtgTblFd)) {
			fgets(szBuf, 256, RtgTblFd);
		}
		if (szBuf[0] != '#') {
			sscanf(szBuf, "%s%s%s", szSwName, szDstName, szToName);

			pszStr = &szSwName[1];
			stRt.ulSwName = strtoul(pszStr, (char **)NULL, 10);

			pszStr = &szDstName[1];
			stRt.ulName = strtoul(pszStr, (char **)NULL, 10);

			GetAddrInf(szToName, szIpAddr, &iCtrlPort, &iDataPort);
			SztoUlulRtgInf(&stRt.ulRtAddr, szIpAddr);
			stRt.ulCtrlPort = iCtrlPort;
			stRt.ulDataPort = iDataPort;

			stRt.ulSwName = htonl(stRt.ulSwName);
			stRt.ulName = htonl(stRt.ulName);
			stRt.ulRtAddr = htonl(stRt.ulRtAddr);
			stRt.ulCtrlPort = htonl(stRt.ulCtrlPort);
			stRt.ulDataPort = htonl(stRt.ulDataPort);
			write(iTcpSkFd, &stRt, sizeof(stRt));
		}
	}

	stSwletCmd.ulCId = SWLET_END;
	stSwletCmd.ulCSize = sizeof(stSwletCmd);
	HtoNstSwletCmd(&stSwletCmd, &stSwletCmd);
	write(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));

	read(iTcpSkFd, &stSwletCmd, sizeof(stSwletCmd));
	fclose(RtgTblFd);
	return OK;
}

int
CloseRtgCtrlSession(iTcpSkFd)
int iTcpSkFd;
{
	close(iTcpSkFd);
	return OK;
}

int
CloseRtgCtrl()
{
	close(giRtgCtrlErrLogFd);
	return OK;
}

