
/*****************************************************************************/
/*                                                                           */
/*   Program : Session Component of Plug-In-able Switching System            */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "SessionComp.h"
#include "ProcComp.h"
/* define global variable */
extern int giErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];
extern struct StPrcTbl astPrcTbl[MAX_PROCESS_NO];

static unsigned int ulSsIdCnt;
struct StSsCmpo astSsCmpo[MAX_SESSION_NO];

/* Session Manager */
int
SessionManager(iTcpPortSkFd, pstSsCtrlData)
int iTcpPortSkFd;
struct StSsCtrlData *pstSsCtrlData;
{
	switch (pstSsCtrlData->ulCcId) {
	case SESSION_CTRL :
		if (SessionCtrl(iTcpPortSkFd, pstSsCtrlData) != ERROR) {
		} else {
			sprintf(gszErrLog, "Error in Session Ctrl.\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			return ERROR;
		}
		break;
	case SESSION_CTRL_ACK :
		sprintf(gszErrLog, "Not implemented SESSION_CTRL_ACK in SessionManager\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	default :
		sprintf(gszErrLog, "Error in SessionManager (ulCcId = %d).\n", pstSsCtrlData->ulCcId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		break;
	}
	return OK;
}

/* Packet Manager */
int
PacketManager(pstPk)
struct StPk *pstPk;
{
	int i;
	struct StPkCmpo *pstPkCmpo;
	struct StPkCmpo *pstLocPkCmpo;

	/*
	printf("SrcId %ld, SsId %ld, Seq %ld.\n", pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId, pstPk->stPkHdr.ulSeqNo);
	*/

	if (pstPk->stPkHdr.ulSeqNo == 0) {
		if (SessionCreate(pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId) == ERROR) {
			sprintf(gszErrLog, "Error in create SessionId(%ld : %ld).\n", pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId);
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			PutMsg(gszErrLog);
			return ERROR;
		}
	}
	if ((i = SsCmpoFind(pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId)) == ERROR) {
		sprintf(gszErrLog, "Error SessionId(%ld : %ld) is not in SessionComponent.\n", pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		PutMsg(gszErrLog);
		return ERROR;
	}
	if ((pstPkCmpo = (struct StPkCmpo *) malloc(sizeof(struct StPkCmpo))) == NULL) {
		sprintf(gszErrLog, "Error in malloc (PacketManager).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	if ((pstPkCmpo->iShmId = shmget(IPC_PRIVATE, sizeof(struct StPkInf), SHM_R | SHM_W)) < 0) {
		sprintf(gszErrLog, "Error in shmget (PacketManager).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		free(pstPkCmpo);
		return ERROR;
	}
	if ((pstPkCmpo->pstPkInf = shmat(pstPkCmpo->iShmId, 0, 0)) == (struct StPkInf *) -1) {
		sprintf(gszErrLog, "Error in shmat(PacketManager).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmctl(pstPkCmpo->iShmId, IPC_RMID, 0);
		free(pstPkCmpo);
		return ERROR;
	}
	pstPkCmpo->pstPkCmpoNxt = NULL;
	memcpy(&(pstPkCmpo->pstPkInf->stPkHdr), &(pstPk->stPkHdr), sizeof(pstPk->stPkHdr));
	pstPkCmpo->pstPkInf->ulHndrId = pstPk->stPkHdr.ulDataId;
	if ((pstPkCmpo->pstPkInf->iShmId = shmget(IPC_PRIVATE, pstPk->stPkHdr.ulPkSize, SHM_R | SHM_W)) < 0) {
		sprintf(gszErrLog, "Error in shmget (PacketManager PacketData).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkCmpo->pstPkInf);
		shmctl(pstPkCmpo->iShmId, IPC_RMID, 0);
		return ERROR;
	}
	if ((pstPkCmpo->pstPkInf->psPkData = shmat(pstPkCmpo->pstPkInf->iShmId, 0,0)) == (unsigned char *) -1) {
		sprintf(gszErrLog, "Error in shmat (PacketManager PacketData).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmctl(pstPkCmpo->pstPkInf->iShmId, IPC_RMID, 0);
		shmdt(pstPkCmpo->pstPkInf);
		shmctl(pstPkCmpo->iShmId, IPC_RMID, 0);
		free(pstPkCmpo);
		return ERROR;
	}
	memcpy(pstPkCmpo->pstPkInf->psPkData, pstPk->sData, pstPk->stPkHdr.ulPkSize);
	
	if (astSsCmpo[i].pstPkCmpo == NULL) {
		astSsCmpo[i].pstPkCmpo = pstPkCmpo;
	} else {
		pstLocPkCmpo = astSsCmpo[i].pstPkCmpo;
		while (pstLocPkCmpo->pstPkCmpoNxt != NULL) {
			pstLocPkCmpo = pstLocPkCmpo->pstPkCmpoNxt;
		}
		pstLocPkCmpo->pstPkCmpoNxt = pstPkCmpo;
	}
	
	shmdt(pstPkCmpo->pstPkInf->psPkData);
	shmdt(pstPkCmpo->pstPkInf);
	
/*
	sprintf(gszErrLog, "receive packet SrcId/SsId/SeqNo(%ld : %ld : %ld).\n", pstPk->stPkHdr.ulSrcId, pstPk->stPkHdr.ulSsId, pstPk->stPkHdr.ulSeqNo);
	PutMsg(gszErrLog);
*/
	sprintf(gszErrLog, "read (PacketManager PacketData).\n");
	write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	return OK;
}

/* Session Create */
int
SessionCreate(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;
	char szStr[50];

	if ((i = SsCmpoFind(ulSrcId, ulSsId))!= ERROR) {
		sprintf(gszErrLog, "Session already exist (%ld : %ld)\n", ulSrcId, ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		PutMsg(gszErrLog);
		return ERROR;
	}
	if ((i = SsCmpoFindEmpty(ulSrcId, ulSsId)) == ERROR) {
		sprintf(gszErrLog, "Error No Empty Session\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	astSsCmpo[i].ulSrcId = ulSrcId;
	astSsCmpo[i].ulSsId = ulSsId;
	bzero(&astSsCmpo[i].stSsInf, sizeof(astSsCmpo[i].stSsInf));
	astSsCmpo[i].stSsInf.bSsAlive = TRUE;
	astSsCmpo[i].stSsInf.iShmId = -1;
	astSsCmpo[i].pstPkCmpo = NULL;

	sprintf(szStr,"New Session Create. (%ld, %ld)\n", ulSrcId, ulSsId);
	PutMsg(szStr);

	return OK;
}

/* Set Session Release */
int
SetSessionRelease(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;
	
	if ((i = SsCmpoFind(ulSrcId, ulSsId)) != ERROR) {
		sprintf(gszErrLog, "Error Session not exist (%ld : %ld)(SetSessionRelease)\n", ulSrcId, ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	astSsCmpo[i].stSsInf.bSsAlive = FALSE;
	return OK;
}

/* Session Release */
int
SessionRelease(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;

	sprintf(gszErrLog, "In SessionRelease. (%ld : %ld) \n", ulSrcId, ulSsId);
	PutMsg(gszErrLog);
	if ((i = SsCmpoFind(ulSrcId, ulSsId)) != ERROR) {
		sprintf(gszErrLog, "Error Session not exist (%ld : %ld)(SessionRelease)\n", ulSrcId, ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	if (astSsCmpo[i].stSsInf.bSsAlive == TRUE) {
		sprintf(gszErrLog, "Error in SessionRelease. (SessionAlive = TRUE)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	PacketFreeR(&astSsCmpo[i].pstPkCmpo);
	if (astSsCmpo[i].stSsInf.iShmId > 0) {
		shmctl(astSsCmpo[i].stSsInf.iShmId, IPC_RMID, 0);
	}
	bzero(&astSsCmpo[i], sizeof(astSsCmpo[i]));
	if (ResourceManager() == ERROR) {
		sprintf(gszErrLog, "Error in Release Resource.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	return OK;
}

/* Packet Free Recursively */
int
PacketFreeR(ppstPkCmpo)
struct StPkCmpo **ppstPkCmpo;
{
	if (*ppstPkCmpo == NULL) {
		return OK;
	} else {
		PacketFreeR(&(*ppstPkCmpo)->pstPkCmpoNxt);
		if (((*ppstPkCmpo)->pstPkInf = shmat((*ppstPkCmpo)->iShmId, 0, 0)) == (struct StPkInf *) -1) {
			sprintf(gszErrLog, "Error in PacketFreeR(shmat).\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		} else {
			shmctl((*ppstPkCmpo)->pstPkInf->iShmId, IPC_RMID, 0);
			shmdt((*ppstPkCmpo)->pstPkInf);
			shmctl((*ppstPkCmpo)->iShmId, IPC_RMID, 0);
		}
		free((*ppstPkCmpo)->pstPkCmpoNxt);
		(*ppstPkCmpo)->pstPkCmpoNxt = NULL;
	}
	return OK;
}

/* Packet Send */
int
PacketSend(pstPkCmpo)
struct StPkCmpo *pstPkCmpo;
{
	int i;
	int iUdpSkFd;
	struct StPk stPk;
	struct sockaddr_in stUdpSkAddr;

	if ((pstPkCmpo->pstPkInf = shmat(pstPkCmpo->iShmId, 0, 0)) == (struct StPkInf *) -1) {
		sprintf(gszErrLog, "Error in shmat.(PacketSend).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	if ((pstPkCmpo->pstPkInf->psPkData = shmat(pstPkCmpo->pstPkInf->iShmId, 0, 0)) == (unsigned char *) -1) {
		sprintf(gszErrLog, "Error in shmat PacketData. (PacketSend).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkCmpo->pstPkInf);
		return ERROR;
	}

	if ((i = SsCmpoFind(pstPkCmpo->pstPkInf->stPkHdr.ulSrcId, pstPkCmpo->pstPkInf->stPkHdr.ulSsId)) == ERROR) {
		sprintf(gszErrLog, "Error in Find SsCmpo (%ld : %ld). (PacketSend).\n", pstPkCmpo->pstPkInf->stPkHdr.ulSrcId, pstPkCmpo->pstPkInf->stPkHdr.ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstPkCmpo->pstPkInf->psPkData);
		shmdt(pstPkCmpo->pstPkInf);
		return ERROR;
	}
	
	iUdpSkFd = socket(AF_INET, SOCK_DGRAM, 0);
	
	bzero(&stUdpSkAddr, sizeof(stUdpSkAddr));
	PacketRouting(&stUdpSkAddr, pstPkCmpo->pstPkInf->stPkHdr.ulRtgInf);

	bzero(&stPk, sizeof(stPk));
	memcpy(stPk.sData, pstPkCmpo->pstPkInf->psPkData, pstPkCmpo->pstPkInf->stPkHdr.ulPkSize);
	memcpy(&stPk.stPkHdr, &(pstPkCmpo->pstPkInf->stPkHdr), sizeof(stPk.stPkHdr));
	shmdt(pstPkCmpo->pstPkInf->psPkData);
	shmdt(pstPkCmpo->pstPkInf);

/*
	sprintf(gszErrLog, "Send SrcId/SsId/SeqNo (%ld:%ld:%ld).(PacketSend)\n", stPk.stPkHdr.ulSrcId, stPk.stPkHdr.ulSsId, stPk.stPkHdr.ulSeqNo);
	write(giErrLogFd, gszErrLog, strlen(gszErrLog));
	PutMsg(gszErrLog);
*/

	HtoNstPkHdr(&stPk.stPkHdr, &stPk.stPkHdr);
	if (sendto(iUdpSkFd, &stPk, sizeof(stPk), 0, &stUdpSkAddr, sizeof(stUdpSkAddr)) == ERROR) {
		sprintf(gszErrLog, "Error in Sending Data.(PacketSend)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iUdpSkFd);
		return ERROR;
	}
	close(iUdpSkFd);
	return OK;
}

/* Packet Free */
int
PacketFree(ppstPkCmpo)
struct StPkCmpo **ppstPkCmpo;
{
	struct StPkCmpo *pstTmpPkCmpo;
	
	pstTmpPkCmpo = *ppstPkCmpo;
	*ppstPkCmpo = pstTmpPkCmpo->pstPkCmpoNxt;
	if ((pstTmpPkCmpo->pstPkInf = shmat(pstTmpPkCmpo->iShmId, 0, 0)) == (struct StPkInf *) -1) {
		sprintf(gszErrLog, "Error in shmat.(PacketFree)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

/*
	printf("Step 3. Src/Ss/Seq(%ld:%ld:%ld)\n", pstTmpPkCmpo->pstPkInf->stPkHdr.ulSrcId, pstTmpPkCmpo->pstPkInf->stPkHdr.ulSsId, pstTmpPkCmpo->pstPkInf->stPkHdr.ulSeqNo);
*/
	if (shmctl(pstTmpPkCmpo->pstPkInf->iShmId, IPC_RMID, 0) < 0) {
		sprintf(gszErrLog, "Error in shmctl.(PacketFree)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		shmdt(pstTmpPkCmpo->pstPkInf);
		return ERROR;
	}

	shmdt(pstTmpPkCmpo->pstPkInf);

	if (shmctl(pstTmpPkCmpo->iShmId, IPC_RMID, 0) < 0) {
		sprintf(gszErrLog, "Error in shmctl. (Packet Free)\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
/*
	printf("SHMRM %ld\n", pstTmpPkCmpo->iShmId);
*/
	
	free(pstTmpPkCmpo);
	return OK;
}

/* Packet Routing */
int
PacketRouting(pstUdpSkAddr, ulRtgInf)
struct sockaddr_in *pstUdpSkAddr;
unsigned long ulRtgInf;
{
	struct {
		unsigned long ulName;
		unsigned long ulRtAddr;
		unsigned long ulCtrlPort;
		unsigned long ulDataPort;
	} stRt;
	char szBuf[MAX_BUFFER_SIZE];
	struct sockaddr_un stClSkAddr;
	struct StSwletCmd stSwletCmd;
	int iIndex;
	int iSkFd;

	if ((iSkFd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
		sprintf(gszErrLog, "Error in OpenSk.(PacketRouting).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	bzero(&stClSkAddr, sizeof(stClSkAddr));
	stClSkAddr.sun_family = AF_LOCAL;
	strcpy(stClSkAddr.sun_path,"Sk");
	strcat(stClSkAddr.sun_path, GetExecName(RT_HNDR));
	if (connect(iSkFd, (struct sockaddr *)&stClSkAddr, sizeof(stClSkAddr)) < 0) {
		sprintf(gszErrLog, "Error in connect Sk. (PacketRouting).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}

	stSwletCmd.ulCId = SWLET_DATA;
	stSwletCmd.ulCSize = sizeof(ulRtgInf);

	write(iSkFd, &stSwletCmd, sizeof(stSwletCmd));
	write(iSkFd, &ulRtgInf, sizeof(ulRtgInf));
	read(iSkFd, &stSwletCmd, sizeof(stSwletCmd));	/* for sync with Processor */
	if (stSwletCmd.ulCId == NOK) {
		sprintf(gszErrLog, "Error in Routing. (PacketRouting).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(iSkFd);
		return ERROR;
	}
	read(iSkFd, &stRt, sizeof(stRt));
	close(iSkFd);

	UltoSzulRtgInf(szBuf, stRt.ulRtAddr);
/*
	sprintf(gszErrLog, "Address : %s, Data Port %ld\n", szBuf, stRt.ulDataPort);
	PutMsg(gszErrLog);
*/
	pstUdpSkAddr->sin_family = AF_INET;
	pstUdpSkAddr->sin_port = htons(stRt.ulDataPort);
	inet_pton(AF_INET, szBuf, &(pstUdpSkAddr->sin_addr));
	return OK;
}

/* Session Find */
int
SsCmpoFind(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;

	for (i = 0; i < MAX_SESSION_NO; i++) {
		if ((astSsCmpo[i].ulSrcId == ulSrcId) && (astSsCmpo[i].ulSsId == ulSsId)) {
			return i;
		}
	}
	return ERROR;
}

/* Session Find Empty */
int
SsCmpoFindEmpty(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;

	for (i = 0; i < MAX_SESSION_NO; i++) {
		if ((astSsCmpo[i].ulSrcId == 0) && (astSsCmpo[i].ulSsId == 0)) {
			return i;
		}
	}
	return ERROR;
}

int
SessionCtrl(iTcpPortSkFd, pstSsCtrlData)
int iTcpPortSkFd;
struct StSsCtrlData *pstSsCtrlData;
{
	long lSize;
	int i;
	int iShmId;
	unsigned char *sBuf;

	SendAck(iTcpPortSkFd, pstSsCtrlData);

	read (iTcpPortSkFd, &lSize, sizeof(lSize));
	lSize = ntohl(lSize);
	if ((i = SsCmpoFind(pstSsCtrlData->ulSrcId, pstSsCtrlData->ulSsId)) == ERROR) {
		sprintf(gszErrLog, "Can't Find Session (%ld : %ld) (SessionCtrl).\n", pstSsCtrlData->ulSrcId, pstSsCtrlData->ulSsId);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}


	iShmId = astSsCmpo[i].stSsInf.iShmId;
	if (iShmId < 0) {
		if ((iShmId = shmget(IPC_PRIVATE, lSize, SHM_R | SHM_W)) < 0) {
			sprintf(gszErrLog, "Error in shmget (SessionCtrl).\n");
			write(giErrLogFd, gszErrLog, strlen(gszErrLog));
			return ERROR;
		}
		astSsCmpo[i].stSsInf.iShmId = iShmId;
	}

	if ((sBuf = shmat(iShmId, 0, 0)) == (unsigned char *) -1) {
		sprintf(gszErrLog, "Error in shmat (SessionCtrl).\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		return ERROR;
	}
	read(iTcpPortSkFd, sBuf, lSize);
	shmdt(sBuf);
	return OK;
}


/* Get Session Infomation Shared Memory Id */
int
GetSsInf(ulSrcId, ulSsId)
unsigned long ulSrcId, ulSsId;
{
	int i;

	if ((i = SsCmpoFind(ulSrcId, ulSsId)) == ERROR) {
		return ERROR;
	}
	return astSsCmpo[i].stSsInf.iShmId;
}

/* Set Session Infomation Shared Memory Id */
int
SetSsInf(ulSrcId, ulSsId, iShmId)
unsigned long ulSrcId, ulSsId;
int iShmId;
{
	int i;

	if ((i = SsCmpoFind(ulSrcId, ulSsId)) == ERROR) {
		return ERROR;
	}
	astSsCmpo[i].stSsInf.iShmId = iShmId;
	return OK;
}
