/*****************************************************************************/
/*                                                                           */
/*   Program : Switchlet Loader of Plug-In-able Switching System             */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include "CommHeader.h"
#include "SwletLdr.h"

int giSwletLdrErrLogFd;
extern char gszErrLog[MAX_LOGSTRING_SIZE];

/* Swlet Loader Initilize */
int
SwletLdrInit(szTxtDstSwIpAddr, iCtrlPort, szTxtSwletName, ulSwletId, pulSsId, piTcpSkFd, piDataFd)
char *szTxtDstSwIpAddr, *szTxtSwletName;
int iCtrlPort;
unsigned long ulSwletId;
unsigned long *pulSsId;
int *piTcpSkFd, *piDataFd;
{
	int i;

	struct sockaddr_in stSsCtrlSkAddr;
	struct StSsCtrlData stSsCtrlData;

	/* open error log file */
	if ((giSwletLdrErrLogFd = open("SwletLdrErr.log", O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		PutSwletLdrSts("Error Log files can not access.");
		return ERROR;
	}

	/* open Swlet file */
	if ((*piDataFd = open(szTxtSwletName, O_RDONLY)) < 0) {
		sprintf(gszErrLog, "Error in Openning DataFile.\n");
		write(giSwletLdrErrLogFd, gszErrLog, strlen(gszErrLog));
		PutSwletLdrSts(gszErrLog);
		close(giSwletLdrErrLogFd);
		return ERROR;
	}

	PutSwletLdrSts("Session Open.");

	/* Open Swlet Ldr session */

	if (OpenSwletLdrSession(szTxtDstSwIpAddr, iCtrlPort, pulSsId, piTcpSkFd) == ERROR) {
		close(giSwletLdrErrLogFd);
		return ERROR;
	}

	return OK;
}

/* Create SwletLdr Session */
int
OpenSwletLdrSession(szTxtDstSwIpAddr, iCtrlPort, pulSsId, piTcpSkFd)
char *szTxtDstSwIpAddr;
int iCtrlPort;
unsigned long *pulSsId;
int *piTcpSkFd;
{
	int i;
	struct sockaddr_in stSsCtrlSkAddr;
	struct StSsCtrlData stSsCtrlData;

	*pulSsId = 0;
	
	*piTcpSkFd = socket(AF_INET, SOCK_STREAM, 0);
	
	bzero(&stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));
	stSsCtrlSkAddr.sin_family = AF_INET;
	stSsCtrlSkAddr.sin_port = htons(iCtrlPort);
	inet_pton(AF_INET, szTxtDstSwIpAddr, &stSsCtrlSkAddr.sin_addr);

	connect(*piTcpSkFd, (struct sockaddr *) &stSsCtrlSkAddr, sizeof(stSsCtrlSkAddr));

	stSsCtrlData.ulCcId = (PROCESS_CREATE);
	stSsCtrlData.ulSsId = 0;
	
	HtoNstSsCtrlData(&stSsCtrlData, &stSsCtrlData);
	if (write(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData)) < 0) {
		sprintf(gszErrLog, "Error in sending Session Create.\n");
		write(giSwletLdrErrLogFd, gszErrLog, strlen(gszErrLog));
		PutSwletLdrSts(gszErrLog);
		close(*piTcpSkFd);
		return ERROR;
	}

	i = read(*piTcpSkFd, &stSsCtrlData, sizeof(stSsCtrlData));
	
	NtoHstSsCtrlData(&stSsCtrlData, &stSsCtrlData);

	if (stSsCtrlData.ulCcId == PROCESS_CREATE_ACK) {
		*pulSsId = stSsCtrlData.ulSsId;
	} else {
		sprintf(gszErrLog, "Error in receiving Prc Create Ack.\n");
		write(giSwletLdrErrLogFd, gszErrLog, strlen(gszErrLog));
		close(*piTcpSkFd);
		return ERROR;
	}

	return OK;
}

/* Release Session */
int
CloseSwletLdrSession(iTcpSkFd, iDataFd)
int iTcpSkFd;
int iDataFd;
{
	close(iTcpSkFd);
	close(iDataFd);
	close(giSwletLdrErrLogFd);
	return OK;
}

/* Send Data to Session */
int
SendSwletToSession(ulSsId, ulSwletId, iTcpSkFd, iDataFd)
unsigned long ulSsId;
unsigned long ulSwletId;
int iTcpSkFd;
int iDataFd;
{
	
	int j;

	struct StPkHdr stPkHdr;
	struct sockaddr_in stSsDataSkAddr;
	unsigned char sBuffer[MAX_BUFFER_SIZE];

	stPkHdr.ulSsId = ulSsId;
	stPkHdr.ulSeqNo = 0;
	stPkHdr.ulPrcId = PROCESS_PACKET;
	stPkHdr.ulRtgInf = 0;
	stPkHdr.ulDataId = ulSwletId;
	stPkHdr.ulPkSize = 0;

	HtoNstPkHdr(&stPkHdr, &stPkHdr);
	write(iTcpSkFd, &stPkHdr, sizeof(stPkHdr));

	while ((j = read(iDataFd, &sBuffer, sizeof(sBuffer))) > 0) {
		if (write(iTcpSkFd, &sBuffer, j) < j) {
			sprintf(gszErrLog, "Error in Sending Data.\n");
			write(giSwletLdrErrLogFd, gszErrLog, strlen(gszErrLog));
		}
	}
	return OK;
}
