
/*****************************************************************************/
/*                                                                           */
/*   Program : X Switch Control Main of Plug-In-able Switching System        */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 11/20/1999                                                    */
/*                                                                           */
/*****************************************************************************/

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include <Xm/Frame.h>

#include "CommHeader.h"

#define szErrLogFileName	"SwCtrl.log"

int giErrLogFd;
char gszErrLog[MAX_LOGSTRING_SIZE];

struct StTrafGen {
	unsigned long ulSrcAddr;
	unsigned long ulSsId;
	unsigned long ulDstAddr;
	unsigned long ulTrafTypeId;
	int iCtrlPort;
	int iDataPort;
	char *szTxtTrafSrc;
	char *szTxtSwNameIpAddr;
};

Widget wTopLevel;
Widget wShSwCtrlMain;
Widget wDlgSwletLdr;
Widget wDlgSwletCtrl;
Widget wDlgCutSwlet;
Widget wDlgTrafGen;
Widget wDlgRtgCtrl;

void
MakeShSwCtrlMain(Widget);

void
MakeDlgSwletLdr(Widget);

void
MakeDlgSwletCtrl(Widget);

void
MakeDlgCutSwlet(Widget);

void
MakeDlgTrafGen(Widget);

void
MakeDlgRtgCtrl(Widget);

void
SwletLoad(Widget, Widget, XmPushButtonCallbackStruct *);

void
SwletCtrl(Widget, Widget, XmPushButtonCallbackStruct *);

void
TrafGen(Widget, Widget, XmPushButtonCallbackStruct *);

void
TrafRcv(Widget, Widget, XmPushButtonCallbackStruct *);

void
RtgCtrl(Widget, Widget, XmPushButtonCallbackStruct *);

void
SwletLdrLoad(Widget, Widget, XmPushButtonCallbackStruct *);

void
CutSwletSendCtrl(Widget, Widget, XmPushButtonCallbackStruct *);

void
TrafGenGenerate(Widget, Widget, XmPushButtonCallbackStruct *);

void
RtgCtrlSend(Widget, Widget, XmPushButtonCallbackStruct *);

void
SwletCtrlSelect(Widget, XtPointer, XmListCallbackStruct *);

void
PutSwletLdrSts(char *);

void
PutCutSwletSts(char *);

void
PutTrafGenSts(char *);

void
PutRtgCtrlSts(char *);

void
RunSwletLdr(char *, int, char *, unsigned long);

void
RunCutSwlet(char *, int, unsigned long, unsigned long, unsigned long);

void *
RunTrafGen(void *);

void
RunRtgCtrl(char *, char *);

main(argc, argv)
int argc;
char *argv[];
{
	XtAppContext app;
	int i;

	if ((giErrLogFd = open(szErrLogFileName, O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error Log files can not access.\n");
		exit(1);
	}
	


	wTopLevel = XtVaAppInitialize(&app, "xSwCtrl", NULL, 0, &argc, argv, NULL, NULL);

	MakeShSwCtrlMain(wTopLevel);

	XtRealizeWidget(wTopLevel);
	XtAppMainLoop(app);

}

void MakeShSwCtrlMain(wTopLevel)
Widget wTopLevel;
{

	char szStr[255];
	XmString xmString;

	Widget wFomSwCtrl, wLblSwCtrlTitle;
	Widget wLblSwlet, wFrmSwlet, wFomSwlet, wBtnSwletLoad, wBtnSwletCtrl;
	Widget wLblTraf, wFrmTraf, wFomTraf, wBtnTrafGen, wBtnTrafRcv;
	Widget wLblRtg, wFrmRtg, wBtnRtgCtrl;

	wFomSwCtrl = XtCreateManagedWidget("wFomSwCtrl",
			xmFormWidgetClass,
			wTopLevel,
			NULL, 0);
	
	xmString = XmStringCreateSimple("X Switch Control Center");
	wLblSwCtrlTitle = XtVaCreateManagedWidget("wLblSwCtrlTitle",
				xmLabelWidgetClass,
				wFomSwCtrl,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwCtrlTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	
	xmString = XmStringCreateSimple("Switchlet");
	wLblSwlet = XtVaCreateManagedWidget("wLblSwlet",
			xmLabelWidgetClass,
			wFomSwCtrl,
			XmNlabelString, xmString,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wLblSwCtrlTitle,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset, 20,
			NULL);
	wFrmSwlet = XtVaCreateManagedWidget("wFrmSwlet",
			xmFrameWidgetClass,
			wFomSwCtrl,
			XmNshadowType, XmSHADOW_ETCHED_IN,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wLblSwlet,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset, 20,
			NULL);
	wFomSwlet = XtVaCreateManagedWidget("wFomSwlet",
			xmFormWidgetClass,
			wFrmSwlet,
			NULL);
	xmString = XmStringCreateSimple("Switchlet Load");
	wBtnSwletLoad = XtVaCreateManagedWidget("wBtnSwletLoad",
			xmPushButtonWidgetClass,
			wFomSwlet,
			XmNlabelString, xmString,
			NULL);
	XtAddCallback(wBtnSwletLoad, XmNactivateCallback, SwletLoad, wFomSwlet);

	xmString = XmStringCreateSimple("Switchlet Ctrl");
	wBtnSwletCtrl = XtVaCreateManagedWidget("wBtnSwletCtrl",
			xmPushButtonWidgetClass,
			wFomSwlet,
			XmNlabelString, xmString,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wBtnSwletLoad,
			NULL);
	XtAddCallback(wBtnSwletCtrl, XmNactivateCallback, SwletCtrl, NULL);


	xmString = XmStringCreateSimple("Traffic");
	wLblTraf = XtVaCreateManagedWidget("wLblTraf",
			xmLabelWidgetClass,
			wFomSwCtrl,
			XmNlabelString, xmString,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wLblSwCtrlTitle,
			XmNleftAttachment, XmATTACH_WIDGET,
			XmNleftWidget, wFrmSwlet,
			XmNleftOffset, 20,
			NULL);
	wFrmTraf = XtVaCreateManagedWidget("wFrmTraf",
			xmFrameWidgetClass,
			wFomSwCtrl,
			XmNshadowType, XmSHADOW_ETCHED_IN,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wLblTraf,
			XmNleftAttachment, XmATTACH_WIDGET,
			XmNleftWidget, wFrmSwlet,
			XmNleftOffset, 20,
			NULL);
	wFomTraf = XtVaCreateManagedWidget("wFomTraf",
			xmFormWidgetClass,
			wFrmTraf,
			NULL);
	xmString = XmStringCreateSimple("Traffic Generate");
	wBtnTrafGen = XtVaCreateManagedWidget("wBtnTrafGen",
			xmPushButtonWidgetClass,
			wFomTraf,
			XmNlabelString, xmString,
			NULL);
	XtAddCallback(wBtnTrafGen, XmNactivateCallback, TrafGen, NULL);

	xmString = XmStringCreateSimple("Traffic Receive");
	wBtnTrafRcv = XtVaCreateManagedWidget("wBtnTrafRcv",
			xmPushButtonWidgetClass,
			wFomTraf,
			XmNlabelString, xmString,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wBtnTrafGen,
			NULL);
	XtAddCallback(wBtnTrafRcv, XmNactivateCallback, TrafRcv, NULL);

	xmString = XmStringCreateSimple("Routing");
	wLblRtg = XtVaCreateManagedWidget("wLblRtg",
			xmLabelWidgetClass,
			wFomSwCtrl,
			XmNlabelString, xmString,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wFrmSwlet,
			XmNtopOffset, 20,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNleftPosition, 40,
			NULL);

	wFrmRtg = XtVaCreateManagedWidget("wFrmRtg",
			xmFrameWidgetClass,
			wFomSwCtrl,
			XmNshadowType, XmSHADOW_ETCHED_IN,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, wLblRtg,
			XmNleftAttachment, XmATTACH_POSITION,
			XmNleftPosition, 40,
			NULL);
	xmString = XmStringCreateSimple("Routing Ctrl");
	wBtnRtgCtrl = XtVaCreateManagedWidget("wBtnRtgCtrl",
			xmPushButtonWidgetClass,
			wFrmRtg,
			XmNlabelString, xmString,
			NULL);
	XtAddCallback(wBtnRtgCtrl, XmNactivateCallback, RtgCtrl, NULL);
}

void
MakeDlgSwletLdr(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanSwletLdr;
	Widget wFomSwletLdr, wLblSwletLdrTitle;
	Widget wLblDstSw, wTxtDstSw;
	Widget wLblSwletName, wTxtSwletName;
	Widget wLblSwletId, wTxtSwletId;
	Widget wBtnLoad;
	Widget wLblSts, wTxtSts;

	wDlgSwletLdr = XtVaCreatePopupShell("wDlgSwletLdr",
			xmDialogShellWidgetClass,
			wTopLevel,
			NULL);

	wPanSwletLdr = XtVaCreateWidget("wPanSwletLdr",
			xmPanedWindowWidgetClass,
			wDlgSwletLdr,
			NULL);

	wFomSwletLdr = XtCreateManagedWidget("wFomSwletLdr",
			xmFormWidgetClass,
			wPanSwletLdr,
			NULL, 0);

	xmString = XmStringCreateSimple("X Switchlet Loader");
	wLblSwletLdrTitle = XtVaCreateManagedWidget("wLblSwletLdrTitle",
				xmLabelWidgetClass,
				wFomSwletLdr,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwletLdrTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Destination Switch : ");
	wLblDstSw = XtVaCreateManagedWidget("wLblDstSw",
			xmLabelWidgetClass,
			wFomSwletLdr,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblDstSw,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletLdrTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtDstSw = XtVaCreateManagedWidget("wTxtDstSw",
			xmTextWidgetClass,
			wFomSwletLdr,
			NULL);
	XtVaSetValues(wTxtDstSw,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletLdrTitle,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblDstSw,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Switchlet Name : ");
	wLblSwletName = XtVaCreateManagedWidget("wLblSwletName",
				xmLabelWidgetClass,
				wFomSwletLdr,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwletName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblDstSw,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSwletName = XtVaCreateManagedWidget("wTxtSwletName",
				xmTextWidgetClass,
				wFomSwletLdr,
				NULL);
	XtVaSetValues(wTxtSwletName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblDstSw,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSwletName,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);


	xmString = XmStringCreateSimple("Switchlet Id : ");
	wLblSwletId = XtVaCreateManagedWidget("wLblSwletId",
			xmLabelWidgetClass,
			wFomSwletLdr,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSwletId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletName,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSwletId = XtVaCreateManagedWidget("wTxtSwletId",
			xmTextWidgetClass,
			wFomSwletLdr,
			NULL);
	XtVaSetValues(wTxtSwletId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletName,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSwletId,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Load");
	wBtnLoad = XtVaCreateManagedWidget("wBtnLoad",
			xmPushButtonWidgetClass,
			wFomSwletLdr,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wBtnLoad,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnLoad, XmNactivateCallback, SwletLdrLoad, wFomSwletLdr);

	xmString = XmStringCreateSimple("Status : ");
	wLblSts = XtVaCreateManagedWidget("wLblSts",
			xmLabelWidgetClass,
			wFomSwletLdr,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnLoad,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSts = XtVaCreateManagedWidget("wTxtSts",
			xmTextWidgetClass,
			wFomSwletLdr,
			NULL);
	XtVaSetValues(wTxtSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnLoad,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSts,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	XtManageChild(wPanSwletLdr);
	XtManageChild(wDlgSwletLdr);

}

void
MakeDlgSwletCtrl(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanSwletCtrl;
	Widget wFomSwletCtrl, wLblSwletCtrlTitle;
	Widget wLblSwletCtrlHead;
	Widget wLstSwletCtrl;
	char szBuf[100];

	wDlgSwletCtrl = XtVaCreatePopupShell("wDlgSwletCtrl",
				xmDialogShellWidgetClass,
				wTopLevel,
				NULL);

	wPanSwletCtrl = XtVaCreateWidget("wPanSwletCtrl",
				xmPanedWindowWidgetClass,
				wDlgSwletCtrl,
				NULL);

	wFomSwletCtrl = XtVaCreateManagedWidget("wFomSwletCtrl",
				xmFormWidgetClass,
				wPanSwletCtrl,
				NULL);

	xmString  = XmStringCreateSimple("X Swtchlet Control Center");
	wLblSwletCtrlTitle = XtVaCreateManagedWidget("wLblSwletCtrlTitle",
				xmLabelWidgetClass,
				wFomSwletCtrl,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwletCtrlTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	sprintf(szBuf, "%15s %15s", "Switchlet", "Controlable");
	xmString = XmStringCreateSimple(szBuf);
	wLblSwletCtrlHead = XtVaCreateManagedWidget("wLblSwletCtrlHead",
				xmLabelWidgetClass,
				wFomSwletCtrl,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwletCtrlHead,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletCtrlTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wLstSwletCtrl = XmCreateScrolledList(wFomSwletCtrl,
				"wLstSwletCtrl",
				NULL, 0);
	XtVaSetValues(XtParent(wLstSwletCtrl),
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwletCtrlHead,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNvisibleItemCount, 10,
		NULL);
	XtManageChild(wLstSwletCtrl);
	XtAddCallback(wLstSwletCtrl, XmNdefaultActionCallback, SwletCtrlSelect, NULL);

	sprintf(szBuf, "%15s %15s         ", "Cut", "Yes");
	xmString = XmStringCreateSimple(szBuf);
	XmListAddItem(wLstSwletCtrl,
		xmString,
		1);

	XtManageChild(wPanSwletCtrl);
	XtManageChild(wDlgSwletCtrl);
}

void
MakeDlgCutSwlet(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanCutSwlet;
	Widget wFomCutSwlet, wLblCutSwletTitle;
	Widget wLblSwName, wTxtSwName;
	Widget wLblCutSize, wTxtCutSize;
	Widget wLblSrcId, wTxtSrcId;
	Widget wLblSsId, wTxtSsId;
	Widget wBtnSendCtrl;
	Widget wLblSts, wTxtSts;

	wDlgCutSwlet = XtVaCreatePopupShell("wDlgCutSwlet",
				xmDialogShellWidgetClass,
				wTopLevel,
				NULL);

	wPanCutSwlet = XtVaCreateWidget("wPanCutSwlet",
				xmPanedWindowWidgetClass,
				wDlgCutSwlet,
				NULL);

	wFomCutSwlet = XtVaCreateManagedWidget("wFomCutSwlet",
				xmFormWidgetClass,
				wPanCutSwlet,
				NULL);

	xmString = XmStringCreateSimple("X Cut Swlet Controller");
	wLblCutSwletTitle = XtVaCreateManagedWidget("wLblCutSwletTitle",
				xmLabelWidgetClass,
				wFomCutSwlet,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblCutSwletTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Switch Name : ");
	wLblSwName = XtVaCreateManagedWidget("wLblSwName",
			xmLabelWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSwName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblCutSwletTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSwName = XtVaCreateManagedWidget("wTxtSwName",
			xmTextWidgetClass,
			wFomCutSwlet,
			NULL);
	XtVaSetValues(wTxtSwName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblCutSwletTitle,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSwName,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Cut Size : ");
	wLblCutSize = XtVaCreateManagedWidget("wLblCutSize",
			xmLabelWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblCutSize,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwName,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtCutSize = XtVaCreateManagedWidget("wTxtCutSize",
			xmTextWidgetClass,
			wFomCutSwlet,
			NULL);
	XtVaSetValues(wTxtCutSize,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwName,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblCutSize,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Source Id : ");
	wLblSrcId = XtVaCreateManagedWidget("wLblSrcId",
			xmLabelWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSrcId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblCutSize,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSrcId = XtVaCreateManagedWidget("wTxtSrcId",
			xmTextWidgetClass,
			wFomCutSwlet,
			NULL);
	XtVaSetValues(wTxtSrcId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblCutSize,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSrcId,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Session Id : ");
	wLblSsId = XtVaCreateManagedWidget("wLblSsId",
			xmLabelWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSsId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSrcId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSsId = XtVaCreateManagedWidget("wTxtSsId",
			xmTextWidgetClass,
			wFomCutSwlet,
			NULL);
	XtVaSetValues(wTxtSsId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSrcId,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSsId,
		XmNrightAttachment,XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Send Ctrl");
	wBtnSendCtrl = XtVaCreateManagedWidget("wBtnSendCtrl",
			xmPushButtonWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wBtnSendCtrl,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSsId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnSendCtrl, XmNactivateCallback, CutSwletSendCtrl, wFomCutSwlet);

	xmString = XmStringCreateSimple("Status : ");
	wLblSts = XtVaCreateManagedWidget("wLblSts",
			xmLabelWidgetClass,
			wFomCutSwlet,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnSendCtrl,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSts = XtVaCreateManagedWidget("wTxtSts",
				xmTextWidgetClass,
				wFomCutSwlet,
				NULL);
	XtVaSetValues(wTxtSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnSendCtrl,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSts,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	XtManageChild(wPanCutSwlet);
	XtManageChild(wDlgCutSwlet);
}

void
MakeDlgTrafGen(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanTrafGen;
	Widget wFomTrafGen, wLblTrafGenTitle;
	Widget wLblSrcAddr, wTxtSrcAddr;
	Widget wLblSsId, wTxtSsId;
	Widget wLblDstAddr, wTxtDstAddr;
	Widget wLblTrafSrc, wTxtTrafSrc;
	Widget wLblTrafTypeId, wTxtTrafTypeId;
	Widget wLblSwName, wTxtSwName;
	Widget wBtnGen;
	Widget wLblSts, wTxtSts;

	wDlgTrafGen = XtVaCreatePopupShell("wDlgTrafGen",
			xmDialogShellWidgetClass,
			wTopLevel,
			NULL);

	wPanTrafGen = XtVaCreateWidget("wPanTrafGen",
			xmPanedWindowWidgetClass,
			wDlgTrafGen,
			NULL);

	wFomTrafGen = XtVaCreateManagedWidget("wFomTrafGen",
			xmFormWidgetClass,
			wPanTrafGen,
			NULL);

	xmString = XmStringCreateSimple("X Traffic Generator");
	wLblTrafGenTitle = XtVaCreateManagedWidget("wLblTrafGenTitle",
				xmLabelWidgetClass,
				wFomTrafGen,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblTrafGenTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Source Addr. : ");
	wLblSrcAddr = XtVaCreateManagedWidget("wLblSrcAddr",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);

	XtVaSetValues(wLblSrcAddr,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafGenTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSrcAddr = XtVaCreateManagedWidget("wTxtSrcAddr",
			xmTextWidgetClass,
			wFomTrafGen,
			NULL);

	XtVaSetValues(wTxtSrcAddr,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafGenTitle,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSrcAddr,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Session Id : ");
	wLblSsId = XtVaCreateManagedWidget("wLblSsId",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSsId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSrcAddr,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSsId = XtVaCreateManagedWidget("wTxtSsId",
			xmTextWidgetClass,
			wFomTrafGen,
			NULL);
	XtVaSetValues(wTxtSsId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSrcAddr,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSsId,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Destination Addr. : ");
	wLblDstAddr = XtVaCreateManagedWidget("wLblDstAddr",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblDstAddr,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSsId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtDstAddr = XtVaCreateManagedWidget("wTxtDstAddr",
			xmTextWidgetClass,
			wFomTrafGen,
			NULL);
	XtVaSetValues(wTxtDstAddr,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSsId,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblDstAddr,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);


	xmString = XmStringCreateSimple("Traffic Source : ");
	wLblTrafSrc = XtVaCreateManagedWidget("wLblTrafSrc",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblTrafSrc,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblDstAddr,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtTrafSrc = XtVaCreateManagedWidget("wTxtTrafSrc",
			xmTextWidgetClass,
			wFomTrafGen,
			NULL);
	XtVaSetValues(wTxtTrafSrc,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblDstAddr,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblTrafSrc,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Switchlet Id : ");
	wLblTrafTypeId = XtVaCreateManagedWidget("wLblTrafTypeId",
				xmLabelWidgetClass,
				wFomTrafGen,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblTrafTypeId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafSrc,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtTrafTypeId = XtVaCreateManagedWidget("wTxtTrafTypeId",
				xmTextWidgetClass,
				wFomTrafGen,
				NULL);
	XtVaSetValues(wTxtTrafTypeId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafSrc,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblTrafTypeId,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("1st Switch Name : ");
	wLblSwName = XtVaCreateManagedWidget("wLblSwName",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSwName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafTypeId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSwName = XtVaCreateManagedWidget("wTxtSwName",
			xmTextWidgetClass,
			wFomTrafGen,
			NULL);
	XtVaSetValues(wTxtSwName,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafTypeId,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSwName,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Generate");
	wBtnGen = XtVaCreateManagedWidget("wBtnGen",
			xmPushButtonWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wBtnGen,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwName,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnGen, XmNactivateCallback, TrafGenGenerate, wFomTrafGen);

	xmString = XmStringCreateSimple("Status : ");
	wLblSts = XtVaCreateManagedWidget("wLblSts",
			xmLabelWidgetClass,
			wFomTrafGen,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnGen,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSts = XtVaCreateManagedWidget("wTxtSts",
				xmTextWidgetClass,
				wFomTrafGen,
				NULL);
	XtVaSetValues(wTxtSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnGen,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSts,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);


	XtManageChild(wPanTrafGen);
	XtManageChild(wDlgTrafGen);
}

void
MakeDlgRtgCtrl(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanRtgCtrl;
	Widget wFomRtgCtrl, wLblRtgCtrlTitle;
	Widget wLblSwInfo, wTxtSwInfo;
	Widget wLblRtgTbl, wTxtRtgTbl;
	Widget wBtnSend;
	Widget wLblSts, wTxtSts;

	wDlgRtgCtrl = XtVaCreatePopupShell("wDlgRtgCtrl",
			xmDialogShellWidgetClass,
			wTopLevel,
			NULL);

	wPanRtgCtrl = XtVaCreateWidget("wPanRtgCtrl",
			xmPanedWindowWidgetClass,
			wDlgRtgCtrl,
			NULL);

	wFomRtgCtrl = XtVaCreateManagedWidget("wFomRtgCtrl",
			xmFormWidgetClass,
			wPanRtgCtrl,
			NULL);

	xmString = XmStringCreateSimple("X Routing Control Center");
	wLblRtgCtrlTitle = XtVaCreateManagedWidget("wLblRtgCtrlTitle",
				xmLabelWidgetClass,
				wFomRtgCtrl,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblRtgCtrlTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Switch Info. File Name : ");
	wLblSwInfo = XtVaCreateManagedWidget("wLblSwInfo",
			xmLabelWidgetClass,
			wFomRtgCtrl,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSwInfo,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblRtgCtrlTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSwInfo = XtVaCreateManagedWidget("wTxtSwInfo",
			xmTextWidgetClass,
			wFomRtgCtrl,
			NULL);
	XtVaSetValues(wTxtSwInfo,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblRtgCtrlTitle,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSwInfo,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Routing Table File Name : ");
	wLblRtgTbl = XtVaCreateManagedWidget("wLblRtgTbl",
			xmLabelWidgetClass,
			wFomRtgCtrl,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblRtgTbl,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwInfo,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtRtgTbl = XtVaCreateManagedWidget("wTxtRtgTbl",
			xmTextWidgetClass,
			wFomRtgCtrl,
			NULL);
	XtVaSetValues(wTxtRtgTbl,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwInfo,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblRtgTbl,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("Send");
	wBtnSend = XtVaCreateManagedWidget("wBtnSend",
			xmPushButtonWidgetClass,
			wFomRtgCtrl,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wBtnSend,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblRtgTbl,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnSend, XmNactivateCallback, RtgCtrlSend, wFomRtgCtrl);

	xmString = XmStringCreateSimple("Status : ");
	wLblSts = XtVaCreateManagedWidget("wLblSts",
			xmLabelWidgetClass,
			wFomRtgCtrl,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnSend,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtSts = XtVaCreateManagedWidget("wTxtSts",
				xmTextWidgetClass,
				wFomRtgCtrl,
				NULL);
	XtVaSetValues(wTxtSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnSend,
		XmNleftAttachment, XmATTACH_WIDGET,
		XmNleftWidget, wLblSts,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	XtManageChild(wPanRtgCtrl);
	XtManageChild(wDlgRtgCtrl);
}

void
SwletLoad(w, wForm, cbs)
Widget w;
Widget wForm;
XmPushButtonCallbackStruct *cbs;
{
	MakeDlgSwletLdr(wTopLevel);
}

void
SwletCtrl(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	MakeDlgSwletCtrl(wTopLevel);
}

void
TrafGen(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	MakeDlgTrafGen(wTopLevel);
}

void
TrafRcv(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	if (fork() == 0) {	/* child */
		system("xTrafRcv");
	}
}

void
RtgCtrl(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	MakeDlgRtgCtrl(wTopLevel);
}


void
SwletLdrLoad(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	Widget wWidget;
	char *szTxtDstSw, *szTxtSwletName, *szTxtSwletId;
	char szTxtDstSwIpAddr[20];
	int iCtrlPort, iDataPort;
	unsigned long ulSwletId;

	wWidget = XtNameToWidget(wFom, "wTxtDstSw");
	szTxtDstSw = XmTextGetString(wWidget);
	if (strcmp("",szTxtDstSw) == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Destination Switch is empty.");
		return;
	}

	if (GetAddrInf(szTxtDstSw, szTxtDstSwIpAddr, &iCtrlPort, &iDataPort) == ERROR) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Destination Switch Ip address unknown.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtSwletName");
	szTxtSwletName = XmTextGetString(wWidget);
	if (strcmp("", szTxtSwletName) == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Swichlet Name is empty.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtSwletId");
	szTxtSwletId = XmTextGetString(wWidget);
	if (strcmp("", szTxtSwletId) == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Swichlet Id is empty.");
		return;
	}
	ulSwletId = strtoul(szTxtSwletId, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSts");
	XmTextSetString(wWidget, "Running Switchlet Load.");

	RunSwletLdr(szTxtDstSwIpAddr, iCtrlPort, szTxtSwletName, ulSwletId);
}

void
CutSwletSendCtrl(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	Widget wWidget;
	char *szTxtSwName, *szTxtCutSize, *szTxtSrcId, *szTxtSsId;
	char szTxtSwNameIpAddr[20];
	int iCtrlPort, iDataPort;
	int ulCutSize, ulSrcId, ulSsId;

	wWidget = XtNameToWidget(wFom, "wTxtSwName");
	szTxtSwName = XmTextGetString(wWidget);
	if (strcmp(szTxtSwName, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Switch Name is empty.");
		return;
	}

	if (GetAddrInf(szTxtSwName, szTxtSwNameIpAddr, &iCtrlPort, &iDataPort) == ERROR) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Switch Name Ip Address is unknown.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtCutSize");
	szTxtCutSize = XmTextGetString(wWidget);
	if (strcmp(szTxtCutSize, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Cut Size is empty.");
		return;
	}
	ulCutSize = strtoul(szTxtCutSize, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSrcId");
	szTxtSrcId = XmTextGetString(wWidget);
	if (strcmp(szTxtSrcId, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Source Id is empty.");
		return;
	}
	if (toupper(szTxtSrcId[0]) != 'K') {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Source Id is not regulary accepted.");
		return;
	}
	szTxtSrcId++; /* Skip one char */
	ulSrcId = strtoul(szTxtSrcId, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSsId");
	szTxtSsId = XmTextGetString(wWidget);
	if (strcmp(szTxtSsId, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Session Id is empty.");
		return;
	}
	ulSsId = strtoul(szTxtSsId, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSts");
	XmTextSetString(wWidget, "Running Cut Swlet Ctrl.");

	RunCutSwlet(szTxtSwNameIpAddr, iCtrlPort, ulCutSize, ulSrcId, ulSsId);

}

void
TrafGenGenerate(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	Widget wWidget;
	char *szTxtSrcAddr, *szTxtSsId, *szTxtDstAddr;
	char *szTxtTrafSrc, *szTxtTrafTypeId, *szTxtSwName;
	char *szTxtSwNameIpAddr;
	unsigned long ulSrcAddr, ulDstAddr, ulSsId, ulTrafTypeId;
	int iCtrlPort, iDataPort;
	static struct StTrafGen stTrafGen;
	pthread_t iTid;

	wWidget = XtNameToWidget(wFom, "wTxtSrcAddr");
	szTxtSrcAddr = XmTextGetString(wWidget);
	if (strcmp(szTxtSrcAddr, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Source Addr is empty.");
		return;
	}

	if (toupper(szTxtSrcAddr[0]) != 'K') {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Source Addr is not regulary accepted.");
		return;
	}
	szTxtSrcAddr++; /* Skip one char */
	ulSrcAddr = strtoul(szTxtSrcAddr, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSsId");
	szTxtSsId = XmTextGetString(wWidget);
	if (strcmp(szTxtSsId, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Session Id is empty.");
		return;
	}
	ulSsId = strtoul(szTxtSsId, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtDstAddr");
	szTxtDstAddr = XmTextGetString(wWidget);
	if (strcmp(szTxtDstAddr, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Destination Addr is empty.");
		return;
	}
	if (toupper(szTxtDstAddr[0]) != 'K') {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Destination Addr is not regulary accepted.");
		return;
	}
	szTxtDstAddr++; /* Skip one char */
	ulDstAddr = strtoul(szTxtDstAddr, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtTrafSrc");
	szTxtTrafSrc = XmTextGetString(wWidget);
	if (strcmp(szTxtTrafSrc, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Traffic Source is empty.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtTrafTypeId");
	szTxtTrafTypeId = XmTextGetString(wWidget);
	if (strcmp(szTxtTrafTypeId, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Traffic Type Id is empty.");
		return;
	}
	ulTrafTypeId = strtoul(szTxtTrafTypeId, (char **)NULL, 10);

	wWidget = XtNameToWidget(wFom, "wTxtSwName");
	szTxtSwName = XmTextGetString(wWidget);
	if (strcmp(szTxtSwName, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Switch Name is empty.");
		return;
	}
	if (GetAddrInf(szTxtSwName, szTxtSwNameIpAddr, &iCtrlPort, &iDataPort) == ERROR) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Switch Name Ip address unknown.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtSts");
	XmTextSetString(wWidget, "Running Traffic Generate.");

	stTrafGen.ulSrcAddr = ulSrcAddr;
	stTrafGen.ulSsId = ulSsId;
	stTrafGen.ulDstAddr = ulDstAddr;
	stTrafGen.szTxtTrafSrc = szTxtTrafSrc;
	stTrafGen.ulTrafTypeId = ulTrafTypeId;
	stTrafGen.szTxtSwNameIpAddr = szTxtSwNameIpAddr;
	stTrafGen.iCtrlPort = iCtrlPort;
	stTrafGen.iDataPort = iDataPort;

	iTid = 0;
	if (pthread_create(&iTid, NULL, RunTrafGen, (void *)(&stTrafGen))) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Error in Create RunTrafGen.");
		return;
	}
	pthread_detach(iTid);
}

void
RtgCtrlSend(w, wFom, cbs)
Widget w;
Widget wFom;
XmPushButtonCallbackStruct *cbs;
{
	Widget wWidget;
	char *szTxtSwInfo, *szTxtRtgTbl;

	wWidget = XtNameToWidget(wFom, "wTxtSwInfo");
	szTxtSwInfo = XmTextGetString(wWidget);
	if (strcmp(szTxtSwInfo, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Switch Info File Name is empty.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtRtgTbl");
	szTxtRtgTbl = XmTextGetString(wWidget);
	if (strcmp(szTxtRtgTbl, "") == 0) {
		wWidget = XtNameToWidget(wFom, "wTxtSts");
		XmTextSetString(wWidget, "Routing Table is empty.");
		return;
	}

	wWidget = XtNameToWidget(wFom, "wTxtSts");
	XmTextSetString(wWidget, "Running Routing Ctrl.");

	RunRtgCtrl(szTxtSwInfo, szTxtRtgTbl);
}

void
SwletCtrlSelect(w, xtpData, cbs)
Widget w;
XtPointer xtpData;
XmListCallbackStruct *cbs;
{

	switch(cbs->item_position) {
	case 1:
		MakeDlgCutSwlet(wTopLevel);
		break;
	default :
		printf("Error Selected Position is %d.\n", cbs->item_position);

	};
}

void
PutSwletLdrSts(szStr)
char *szStr;
{
	Widget wWidget;

	wWidget = XtNameToWidget(wDlgSwletLdr, "wPanSwletLdr");
	wWidget = XtNameToWidget(wWidget, "wFomSwletLdr");
	wWidget = XtNameToWidget(wWidget, "wTxtSts");
	XmTextSetString(wWidget, szStr);
}

void
PutCutSwletSts(szStr)
char *szStr;
{
	Widget wWidget;

	wWidget = XtNameToWidget(wDlgCutSwlet, "wPanCutSwlet");
	wWidget = XtNameToWidget(wWidget, "wFomCutSwlet");
	wWidget = XtNameToWidget(wWidget, "wTxtSts");
	XmTextSetString(wWidget, szStr);
}

void
PutTrafGenSts(szStr)
char *szStr;
{
	Widget wWidget;

	wWidget = XtNameToWidget(wDlgTrafGen, "wPanTrafGen");
	wWidget = XtNameToWidget(wWidget, "wFomTrafGen");
	wWidget = XtNameToWidget(wWidget, "wTxtSts");
	XmTextSetString(wWidget, szStr);
}

void
PutRtgCtrlSts(szStr)
char *szStr;
{
	Widget wWidget;

	wWidget = XtNameToWidget(wDlgRtgCtrl, "wPanRtgCtrl");
	wWidget = XtNameToWidget(wWidget, "wFomRtgCtrl");
	wWidget = XtNameToWidget(wWidget, "wTxtSts");
	XmTextSetString(wWidget, szStr);
}

void
RunSwletLdr(szTxtDstSwIpAddr, iCtrlPort, szTxtSwletName, ulSwletId)
char *szTxtDstSwIpAddr, *szTxtSwletName;
int iCtrlPort;
unsigned long ulSwletId;
{
	unsigned long ulSsId;
	int iTcpSkFd;
	int iDataFd;

	if (SwletLdrInit(szTxtDstSwIpAddr, iCtrlPort, szTxtSwletName, ulSwletId, &ulSsId, &iTcpSkFd, &iDataFd) == ERROR) {
		return;
	}

	PutSwletLdrSts("Send Swlet.");


	SendSwletToSession(ulSsId, ulSwletId, iTcpSkFd, iDataFd);

	CloseSwletLdrSession(iTcpSkFd, iDataFd);
	PutSwletLdrSts("Session Closed");
}

void
RunCutSwlet(szTxtSwNameIpAddr, iCtrlPort, ulCutSize, ulSrcId, ulSsId)
char *szTxtSwNameIpAddr;
int iCtrlPort;
unsigned long ulCutSize, ulSrcId, ulSsId;
{
	int iTcpSkFd;

	if (CutSwletInit(szTxtSwNameIpAddr, iCtrlPort, ulSrcId, ulSsId, &iTcpSkFd) == ERROR) {
		return;
	}

	PutCutSwletSts("Send Control Signal");

	SendCutSwletCtrlToSession(szTxtSwNameIpAddr, ulCutSize, iTcpSkFd);

	CloseCutSwletSession(iTcpSkFd);
	PutCutSwletSts("Control Session Closed.");
}

void *
RunTrafGen(arg)
void *arg;
{
	int iDataFd;
	struct StTrafGen *pstTrafGen;

	pstTrafGen = (struct StTrafGen *)arg;
	if (TrafGenInit(pstTrafGen->szTxtTrafSrc, &iDataFd) == ERROR) {
		return;
	}

	PutTrafGenSts("Sending Data");

	SendTrafGenDataToSession(pstTrafGen->szTxtSwNameIpAddr,
		pstTrafGen->ulSrcAddr,
		pstTrafGen->ulSsId,
		pstTrafGen->ulDstAddr,
		pstTrafGen->ulTrafTypeId,
		iDataFd,
		pstTrafGen->iCtrlPort,
		pstTrafGen->iDataPort);

	PutTrafGenSts("Data Session Closed");
	pthread_exit(NULL);
	return (void *)NULL;
}

void
RunRtgCtrl(szTxtSwInfo, szTxtRtgTbl)
char *szTxtSwInfo, *szTxtRtgTbl;
{
	FILE *SwInfoFd;
	char szStr[256];
	char szSwName[50];
	unsigned long ulSsId;
	int iTcpSkFd;

	if (RtgCtrlInit(szTxtSwInfo, &SwInfoFd) == ERROR) {
		return;
	}

	PutRtgCtrlSts("Send Routing Tbl.");

	while (!feof(SwInfoFd)) {
		fgets(szStr, 256, SwInfoFd);
		while ((szStr[0] == '#') && !feof(SwInfoFd)) {
			fgets(szStr, 256, SwInfoFd);
		}
		if (szStr[0] != '#') {
			sscanf(szStr, "%s", szSwName);

			PutRtgCtrlSts("Session Open.");

			if (OpenRtgCtrlSession(szSwName, &ulSsId, &iTcpSkFd) == ERROR) {
				CloseRtgCtrl();
				fclose(SwInfoFd);
				return;
			}

			PutRtgCtrlSts("Send Data.");

			SendRtgCtrlToSession(szTxtRtgTbl, iTcpSkFd);

			CloseRtgCtrlSession(iTcpSkFd);

			PutRtgCtrlSts("Session Closed.");
		}
	}

	CloseRtgCtrl();
}
