
/*****************************************************************************/
/*                                                                           */
/*   Program : X Switch Main of Plug-In-able Switching System                */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 9/24/1999                                                     */
/*                                                                           */
/*****************************************************************************/

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>

#include "CommHeader.h"
#include "ManComp.h"
#include "ProcComp.h"

#define	VER	1.0
#define SysIdFileName	"Sys.Id"
#define szSwErrLogFileName	"Sw.log"

int giErrLogFd;
char gszErrLog[MAX_LOGSTRING_SIZE];

int giCtrlPort, giDataPort;

Widget wTopLevel;
Widget wShSwMain, wDlgSwSts, wDlgLdSwlet;
static Widget wTxtSwSts;
Widget wLstLdSwlet;

int iSwStsPos;

void
MakeShSwMain(Widget, unsigned long);

void
MakeDlgSwSts(Widget);

void
MakeDlgLdSwlet(Widget);

void
ViewSwlet(Widget, Widget, XmPushButtonCallbackStruct *);

void
ViewMsg(Widget, Widget, XmPushButtonCallbackStruct *);

main(argc, argv)
int argc;
char *argv[];
{
	XtAppContext app;
	int i;
	unsigned long ulSysId;
	pthread_t iTid;
	FILE *SysIdFd;
	char szSwName[50], szSwIpAddr[50];

	/* GetSystem Identification */	
	if ((SysIdFd = fopen(SysIdFileName, "r")) == NULL) {
		printf("Error. Can't open System Id file.\n");
		exit(1);
	}
	fscanf(SysIdFd, "%ld", &ulSysId);
	fclose(SysIdFd);

	sprintf(szSwName, "K%04ld", ulSysId);
	if (GetAddrInf(szSwName, szSwIpAddr, &giCtrlPort, &giDataPort) == ERROR) {
		printf("Can't Find CtrlPort and DataPort. %s\n", szSwName);
		exit(1);
	}

	if ((giErrLogFd = open(szSwErrLogFileName, O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error Log files can not access.\n");
		exit(1);
	}
	


	wTopLevel = XtVaAppInitialize(&app, "xSwMain", NULL, 0, &argc, argv, NULL, NULL);

	MakeShSwMain(wTopLevel, ulSysId);

	iTid = 0;
	if (pthread_create(&iTid, NULL, ManagerSupervisor, (void *)(NULL))) {
		sprintf(gszErrLog, "Error in Create ManagerSupervisor.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		exit(0);
	}
	pthread_detach(iTid);

	iTid++;
	if (pthread_create(&iTid, NULL, ProcessSupervisor, (void *)(NULL))) {
		sprintf(gszErrLog, "Error in Create ProcessSupervisor.\n");
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		close(giErrLogFd);
		exit(0);
	}
	pthread_detach(iTid);

	XtRealizeWidget(wTopLevel);
	XtAppMainLoop(app);

}

void MakeShSwMain(wTopLevel, ulSysId)
Widget wTopLevel;
unsigned long ulSysId;
{

	char szStr[255];
	XmString xmString;

	Widget wFomSwMain, wLblSwTitle;
	Widget wLblSwId, wLblSwVer;
	Widget wBtnLdSwlet, wBtnSwSts;

	wFomSwMain = XtCreateManagedWidget("wFomSwMain",
			xmFormWidgetClass,
			wTopLevel,
			NULL, 0);
	
	xmString = XmStringCreateSimple("Plug-In-able Switching System");
	wLblSwTitle = XtVaCreateManagedWidget("wLblSwTitle",
				xmLabelWidgetClass,
				wFomSwMain,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblSwTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	
	sprintf(szStr, "Switching System Id. : K%04ld", ulSysId);
	xmString = XmStringCreateSimple(szStr);
	wLblSwId = XtVaCreateManagedWidget("wLblSwId",
				xmLabelWidgetClass,
				wFomSwMain,
				XmNlabelString, xmString,
				XmNheight, 30,
				NULL);
	XtVaSetValues(wLblSwId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	sprintf(szStr, "Switching System Ver. : %.2f", VER);
	xmString = XmStringCreateSimple(szStr);
	wLblSwVer = XtVaCreateManagedWidget("wLblSwVer",
				xmLabelWidgetClass,
				wFomSwMain,
				XmNlabelString, xmString,
				XmNheight, 30,
				NULL);
	XtVaSetValues(wLblSwVer,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwId,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	xmString = XmStringCreateSimple("View Loaded Switchlets");
	wBtnLdSwlet = XtVaCreateManagedWidget("wBtnLdSwlet",
				xmPushButtonWidgetClass,
				wFomSwMain,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wBtnLdSwlet,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwVer,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnLdSwlet, XmNactivateCallback, ViewSwlet, wFomSwMain);

	xmString = XmStringCreateSimple("View Switch Status Messages");
	wBtnSwSts = XtVaCreateManagedWidget("wBtnSwSts",
				xmPushButtonWidgetClass,
				wFomSwMain,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wBtnSwSts,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wBtnLdSwlet,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);
	XtAddCallback(wBtnSwSts, XmNactivateCallback, ViewMsg, wFomSwMain);

}

void
MakeDlgSwSts(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;

	Widget wPanSwSts, wFomSwSts, wLblSwStsTitle;

/*
	wDlgSwSts = XtVaAppCreateShell(NULL,
			"wDlgSwSts",
			topLevelShellWidgetClass,
			XtDisplay(wTopLevel),
			NULL);
	if (wDlgSwSts) {
		printf("wDlgSwSts already Exists.\n");
		XtManageChild(wDlgSwSts);
		return;
	}
*/
	wDlgSwSts = XtVaCreatePopupShell("wDlgSwSts",
			xmDialogShellWidgetClass,
			wTopLevel,
			NULL);

	wPanSwSts = XtVaCreateWidget("wPanSwSts",
			xmPanedWindowWidgetClass,
			wDlgSwSts,
			NULL);

	wFomSwSts = XtCreateManagedWidget("wFomSwSts",
			xmFormWidgetClass,
			wPanSwSts,
			NULL, 0);

	xmString = XmStringCreateSimple("Switch Status Messages");
	wLblSwStsTitle = XtVaCreateManagedWidget("wLblSwStsTitle",
				xmLabelWidgetClass,
				wFomSwSts,
				XmNlabelString, xmString,
				XmNtopAttachment, XmATTACH_FORM,
				XmNleftAttachment, XmATTACH_FORM,
				NULL);

	wTxtSwSts = XmCreateScrolledText(wFomSwSts,
			"wTxtSwSts",
			NULL, 0);
	XtVaSetValues(wTxtSwSts,
		XmNrows, 10,
		XmNcolumns, 80,
		XmNeditable, False,
		XmNeditMode, XmMULTI_LINE_EDIT,
		NULL);
	XtVaSetValues(XtParent(wTxtSwSts),
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSwStsTitle,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_FORM,
		NULL);
	XtManageChild(wTxtSwSts);

	XtManageChild(wPanSwSts);
	iSwStsPos = 0;
}

void
MakeDlgLdSwlet(wTopLevel)
Widget wTopLevel;
{
	char szStr[255];
	XmString xmString;
	
	Widget wPanLdSwlet, wFomLdSwlet, wLblLdSwletTitle;
	
	wDlgLdSwlet = XtVaCreatePopupShell("wDlgLdSwlet",
			xmDialogShellWidgetClass,
			wTopLevel,
			NULL);

	wPanLdSwlet = XtVaCreateWidget("wPanLdSwlet",
			xmPanedWindowWidgetClass,
			wDlgLdSwlet,
			NULL);

	wFomLdSwlet = XtCreateManagedWidget("wFomLdSwlet",
			xmFormWidgetClass,
			wPanLdSwlet,
			NULL, 0);

	xmString = XmStringCreateSimple("Loaded Switchlets");
	wLblLdSwletTitle = XtVaCreateManagedWidget("wLblLdSwletTitle",
				xmLabelWidgetClass,
				wFomLdSwlet,
				XmNlabelString, xmString,
				XmNtopAttachment, XmATTACH_FORM,
				XmNleftAttachment, XmATTACH_FORM,
				NULL);

	wLstLdSwlet = XmCreateScrolledList(wFomLdSwlet,
			"wLstLdSwlet",
			NULL, 0);
	XtVaSetValues(XtParent(wLstLdSwlet),
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblLdSwletTitle,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_FORM,
		NULL);
	XtManageChild(wLstLdSwlet);

	XtManageChild(wPanLdSwlet);
}

void
ViewSwlet(w, Form, cbs)
Widget w;
Widget Form;
XmPushButtonCallbackStruct *cbs;
{
	int i,j;
	char szStr[500];
	char szTmp[50];
	XmString xmString;

	MakeDlgLdSwlet(wTopLevel);
	XtManageChild(wDlgLdSwlet);

	szStr[0] = '\0';
	GetExecNameList(szStr);

	i = 0;
	while ( (i < 500) && (szStr[i] != '\0') ) {
		j = 0;
		while ((i < 500) && (szStr[i]!= '\n') && (szStr[i] != '\0'))
			szTmp[j++] = szStr[i++];
		szTmp[j] = '\0';
		if (szStr[i] = '\n')
			i++;
		xmString = XmStringCreateSimple(szTmp);
		XmListAddItem(wLstLdSwlet,
			xmString,
			1);
	}
}

void
ViewMsg(w, Form, cbs)
Widget w;
Widget Form;
XmPushButtonCallbackStruct *cbs;
{
	MakeDlgSwSts(wTopLevel);
	XtManageChild(wDlgSwSts);
}

void
PutMsg(szMsg)
char *szMsg;
{
/*
	printf("%s\n", szMsg);
*/
	if (wTxtSwSts) {
		XmTextInsert(wTxtSwSts,
			iSwStsPos,
			szMsg);
		iSwStsPos += strlen(szMsg);
	} else {
/*
		printf("%s", szMsg);
*/
	}
}

