/*****************************************************************************/
/*                                                                           */
/*   Program : X Traffic Receiver Main of Plug-In-able Switching System      */
/*   Author  : Seung-su Yang                                                 */
/*   Date    : 11/20/1999                                                    */
/*                                                                           */
/*****************************************************************************/

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/DialogS.h>
#include <Xm/List.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include <Xm/Frame.h>

#include "CommHeader.h"

#define szErrLogFileName	"TrafRcv.log"
#define szDataFileName		"RcvData.log"
#define SysIdFileName		"Sys.Id"

int giErrLogFd;
char gszErrLog[MAX_LOGSTRING_SIZE];
unsigned long ulSysId;

int giCtrlPort, giDataPort;

Widget wTopLevel;
static Widget wTxtTrafSts;

void
MakeMainWindow(Widget);

void *
RcvData(void *);

main(argc, argv)
int argc;
char *argv[];
{
	XtAppContext app;
	int i;
	int iDataFd;
	pthread_t iThread;
	FILE *SysIdFd;
	char szName[50], szIpAddr[50];

	/* Get System Identification */
	if ((SysIdFd = fopen(SysIdFileName, "r")) == NULL) {
		printf("Error. Can't open System Id file.\n");
		exit(1);
	}
	fscanf(SysIdFd, "%ld", &ulSysId);
	fclose(SysIdFd);

	sprintf(szName, "K%04ld", ulSysId);
	if (GetAddrInf(szName, szIpAddr, &giCtrlPort, &giDataPort) == ERROR) {
		printf("Can't Find CtrlPort and DataPort.%s\n", szName);
		exit(1);
	}

	if ((giErrLogFd = open(szErrLogFileName, O_WRONLY | O_APPEND | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error Log files can not access.\n");
		exit(1);
	}

	if ((iDataFd = open(szDataFileName, O_WRONLY | O_CREAT | O_SYNC | O_DSYNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1) {
		printf("Error in Open Data File.\n");
		exit(1);
	}
	


	wTopLevel = XtVaAppInitialize(&app, "xTrafRcv", NULL, 0, &argc, argv, NULL, NULL);

	MakeMainWindow(wTopLevel);

	XtRealizeWidget(wTopLevel);

	if (pthread_create(&iThread, NULL, RcvData, (void *)(&iDataFd))) {
		sprintf(gszErrLog, "Error in Create Handler (thread index %d)\n", iThread);
		write(giErrLogFd, gszErrLog, strlen(gszErrLog));
		exit(1);
	}
	pthread_detach(iThread);
	XtAppMainLoop(app);
}

void MakeMainWindow(wTopLevel)
Widget wTopLevel;
{

	char szStr[255];
	XmString xmString;

	Widget wFomTrafRcv, wLblTrafRcvTitle, wLblSysId;

	wFomTrafRcv = XtCreateManagedWidget("wFomTrafRcv",
			xmFormWidgetClass,
			wTopLevel,
			NULL, 0);
	
	xmString = XmStringCreateSimple("X Traffic Receiver");
	wLblTrafRcvTitle = XtVaCreateManagedWidget("wLblTrafRcvTitle",
				xmLabelWidgetClass,
				wFomTrafRcv,
				XmNlabelString, xmString,
				NULL);
	XtVaSetValues(wLblTrafRcvTitle,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	sprintf(szStr,"System Id K%04ld", ulSysId);
	xmString = XmStringCreateSimple(szStr);
	wLblSysId = XtVaCreateManagedWidget("wLblSysId",
			xmLabelWidgetClass,
			wFomTrafRcv,
			XmNlabelString, xmString,
			NULL);
	XtVaSetValues(wLblSysId,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblTrafRcvTitle,
		XmNleftAttachment, XmATTACH_FORM,
		NULL);

	wTxtTrafSts = XmCreateScrolledText(wFomTrafRcv,
			"wTxtTrafSts",
			NULL, 0);
	XtVaSetValues(wTxtTrafSts,
		XmNrows, 10,
		XmNcolumns, 40,
		XmNeditable, False,
		XmNeditMode, XmMULTI_LINE_EDIT,
		NULL);
	XtVaSetValues(XtParent(wTxtTrafSts),
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, wLblSysId,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_FORM,
		NULL);
	XtManageChild(wTxtTrafSts);
}

void
PutTrafSts(szStr)
char *szStr;
{
	static iTxtPos = 0;

	if (wTxtTrafSts) {
		XmTextInsert(wTxtTrafSts, iTxtPos, szStr);
		iTxtPos += strlen(szStr);
	}
}

void *
RcvData(arg)
void *arg;
{
	int iFd;

	iFd = (int)arg;

	ReceiveDataFromSession(iFd);
	close(iFd);
	pthread_exit(NULL);
	return (void *)NULL;
}

